#!/usr/bin/env python

# Import modules
import sys, string, os
import numpy as np
import yaml
import json
import glob
import casacore.tables as tables
import logging


from astropy.time import Time
from astropy.coordinates import SkyCoord, EarthLocation, AltAz
from astropy.io import fits, ascii
from astropy import units as u
from astropy.table import Table, Column, MaskedColumn


import rfinder


class rfi_stats:



    def __init__(self):
        
        self.logger = logging.getLogger('log-rfinder.log')
        #self.logger.setLevel(logging.INFO)

        #fh = logging.FileHandler('log-rfinder.log')
        #fh.setLevel(logging.INFO)

        #ch = logging.StreamHandler()
        #ch.setLevel(logging.WARNING)

        #formatter = logging.Formatter('%(levelname)s - %(filename)s - %(message)s')
        #fh.setFormatter(formatter)
        #ch.setFormatter(formatter)

        #self.logger.addHandler(ch)
        #self.logger.addHandler(fh)


    def make_psf(self,cfg_par) :
        '''
        use wsclean to predict the psf of the observation
        '''
        command1 = '''wsclean -name psfonly -mem 100 -no-dirty -weight natural -super-weight 1.0'''
        command2 = '''-weighting-rank-filter-size 16 -size 512 512 -scale 3.0asec -channels-out 1'''
        command3 = ''' -grid-mode kb -kernel-size 7 -oversampling 63 -make-psf-only -pol xx -intervals-out 1'''
        command4 = ''' -data-column DATA -gain 0.1 -mgain 1.0 -multiscale-scale-bias 0.6 -fit-beam -no-fit-beam '''+cfg_par['general']['rfidir']

        command = command1+command2+command3+command4

        os.system(command)

        logging.info("\tPSF found\t")

        return 0


    def time_chunk(self,cfg_par):

        self.logger.info("\t ...  Observing time Info ... \n")
        self.msfile = cfg_par['general']['msfullpath']
        
        t=tables.table(self.msfile)
        self.time = t.getcol('TIME')
        t.close()

        starttime= self.time[0]
        
        endtime=self.time[-1]
        time_chunk = float(cfg_par['rfi']['chunks']['time_step'])*60.
        
        times=np.arange(starttime,endtime+time_chunk*1.,time_chunk)
        
        cfg_par['rfi']['times'] = times
        startdate=Time(starttime/3600./24.,format='mjd',scale='utc')
        cfg_par['rfi']['startdate'] = startdate
        startdate.format='iso' 
        startdate.subformat='date_hm'       

        enddate=Time(endtime/3600./24.,format='mjd',scale='utc')
        cfg_par['rfi']['enddate'] = enddate

        enddate.format='iso'        
        enddate.subformat='date_hm'       


        self.logger.info('\t Start date: {0:%y}{0:%b}{0:%d}:{0:%X}'.format(startdate.datetime))
        self.logger.info('\t End date  : {0:%y}{0:%b}{0:%d}:{0:%X} \n\n'.format(enddate.datetime))
        
        return times,startdate,enddate

    def baseline_stats(self,cfg_par):

        self.logger.info("\t ... Baseline info ... \n")


        baseline_cutoff = float(cfg_par['rfi']['baseline_cut'])
        lenghts = np.array([cfg_par['rfi']['baseline_lenghts']])+0.
        index_baselines = (np.abs(lenghts - baseline_cutoff)).argmin()

        nrBaseline = cfg_par['rfi']['number_baseline']
        self.logger.info('\t\t Maximum baseline length     m = '+ str(np.round(lenghts[0][-1],0)))
        self.logger.info('\t\t Minimum baseline length     m = '+ str(np.round(lenghts[0][0],0)))
        self.logger.info('\t\t Total number of baselines     = '+ str(nrBaseline))


        num_short = index_baselines+1
        num_long = nrBaseline-num_short
        cfg_par['rfi']['num_short'] = num_short 
        cfg_par['rfi']['num_long'] = num_long

        self.logger.info('\t\t Number of baselines < {0:d} m = {1:d}'.format(cfg_par['rfi']['baseline_cut'],int(num_short)))
        self.logger.info('\t\t Number of baselines > {0:d} m = {1:d}\n'.format(cfg_par['rfi']['baseline_cut'],int(num_long)))


        #outdir = cfg_par['general']['rfidir']
        #basefile = outdir+str(cfg_par['rfi']['telescope'])+'_baselines.txt'

        #if os.path.exists(basefile) == False:
        #    ascii.write(baseline_array, overwrite=True)

        #self.logger.info("\t ... Baselines saved to file ... \n")

        return 0

    def predict_noise(self,cfg_par,channelWidths,interval,flag):
        '''
        
        Predict expected natural rms adding up all unflaged integrations and polarisations

        this module is takend from noisy.py [https://github.com/paoloserra/noisy]

        '''
        self.logger.info("\t ...  Predicting natural r.m.s. ... \n")
        self.msfile = cfg_par['general']['msfullpath']
        
        # Derive quantities
        kB=1380.6  

        STOKES = ['xx','yy','XX','YY','xy','yx','XY','YX']
        if cfg_par['rfi']['polarization'] in STOKES:                                # Boltzmann constant (Jy m^2 / K)
            polnum = 1
        else:
            polnum = 2
        cfg_par['rfi']['polnum'] = polnum

        tele= cfg_par['general']['telescope']

        if tele == 'meerkat' or tele == 'MeerKAT' or tele == 'meerKAT' or tele == 'meer':
            tsyseff = 30.             # Tsys/eff(K)
            diam = 13.5               # m
        elif tele == 'apertif' or tele == 'Apertif' or tele == 'APERTIF' or tele == 'wsrt':
            tsyseff = 93.
            diam = 25.
        else:
            self.logger.error('\t Telescope not known or not specified, will use WSRT instead')
            tsyseff = 93.
            diam = 25.

        Aant=np.pi*(diam/2)**2                      # collecting area of 1 antenna (m^2)
        SEFD=2*kB*tsyseff/Aant                  # frequency independent system equivalent flux density (Jy)
        
        # Print assumptions
        self.logger.info('\t Assumptions on {0:s} telescope'.format(tele))
        self.logger.info('\t\tDish diameter = {0:.1f} m'.format(diam))
        self.logger.info('\t\t ... and SEFD = {0:0f} Jy'.format(SEFD))
        self.logger.info('\t\t ... and Tsys = {0:.1f} K'.format(tsyseff))

        self.logger.info('\t Properties of observation'.format(tele))
        nrBaseline = cfg_par['rfi']['number_baseline']
        self.logger.info('\t\t Total number of baselines = '+ str(nrBaseline))
        self.logger.info('\t\t Total number of channels = '+ str(channelWidths.shape[1]))
        cfg_par['rfi']['total_channels'] = channelWidths.shape[1]
        cfg_par['rfi']['exptime'] = interval.sum()/nrBaseline/3600.
        self.logger.info('\t\t Observing time on source = {0:.5f} h ({1:d} polarisations)\n'.format(cfg_par['rfi']['exptime'],polnum))

        rms=np.sqrt(2)*kB*tsyseff/Aant/np.sqrt(channelWidths*interval.sum()*polnum)

        if len(rms.shape)==2 and rms.shape[0]==1: rms=rms[0]

        self.logger.info('\t Stokes I natural r.m.s.       = {0:.3e} mJy/b '.format(np.nanmedian(rms*1e3)))

        cfg_par['rfi']['theo_rms'] = rms
        
        self.logger.info("\t ... Natural r.m.s. predicted ... \n")


        return 0


    def alt_az(self,cfg_par,time):

        self.logger.info("\t ... Altitude/Azimuth info ... \n")

        #open miriad observation
        tele= cfg_par['general']['telescope']
        coord = cfg_par['rfi']['coords']

        if tele == 'meerkat' or tele == 'MeerKAT' or tele == 'meerKAT' or tele == 'meer':
            telescope = EarthLocation(lat= -30.712856*u.deg, lon=21.44377374*u.deg, height=1051.0*u.m)        
        elif tele == 'apertif' or tele == 'Apertif' or tele == 'APERTIF' or tele == 'wsrt':
            # Set position of Westerbork (source: http://www.sonel.org/spip.php?page=gps&idStation=884)
            telescope = EarthLocation(lat= 52.91460037*u.deg, lon=6.60449982*u.deg, height=82.2786*u.m)        
        else:
            self.logger.error('\t Observing location unknown, impossible to estimate Alt/Az')
        
        time = Time(time/3600./24.,format='mjd',scale='utc')
        frame = AltAz(obstime=time, location=telescope)
        obs_altaz = coord.transform_to(frame)
        cfg_par['rfi']['altaz'] = obs_altaz
       
        self.logger.info(('\t\t ... Altitude = {0:s}').format(cfg_par['rfi']['altaz'].alt))
        self.logger.info(('\t\t ... Azimuth = {0:s}\n').format(cfg_par['rfi']['altaz'].az))

        self.logger.info("\t ... Alt/Az done ... \n")

        return obs_altaz


import os, string, sys, glob
import numpy as np
from astropy.io import fits as fits
from astropy import units as u
from jinja2 import FileSystemLoader, Environment
import shutil
import rfinder_stats as rfi_stats 
rfiST = rfi_stats.rfi_stats()

import logging
logger = logging.getLogger('log-rfinder.log')
#logger.setLevel(logging.INFO)

#fh = logging.FileHandler('log-rfinder.log')
#fh.setLevel(logging.INFO)

#ch = logging.StreamHandler()
#ch.setLevel(logging.WARNING)

#formatter = logging.Formatter('%(levelname)s - %(filename)s - %(message)s')
#fh.setFormatter(formatter)
#ch.setFormatter(formatter)

#logger.addHandler(ch)
#logger.addHandler(fh)


def write_freq_base(cfg_par,rms,time_step=-1) :
    '''
    
    Writes an image.fits of visibilities ordered by frequency, baseline.
    Baselines are ordered by their length.
    
    '''
    logger.info('\t ... Wrtiting RFI by frequency and baseline ...')


    #reverse frequencies if going from high-to-low         
    if time_step != -1:
        outputdir = cfg_par['general']['rfitimedir']
        time_tmp = int(float(cfg_par['rfi']['chunks']['time_step'])*time_step)
        if time_tmp == 0:
            time_name = '00'+str(time_tmp)+'m'
        elif time_tmp <100:
            time_name = '0'+str(time_tmp)+'m'
        else:
            time_name= str(time_tmp)+'m'    
    else:
        outputdir = str(cfg_par['general']['rfidir'])
        time_name = 'full' 

    if cfg_par['rfi']['RFInder_mode'] == 'use_flags':
        rfi_freq_base =outputdir+'flags_base_'+time_name+'.fits'
    if cfg_par['rfi']['RFInder_mode']== 'rms_clip':
        rfi_freq_base =outputdir+'rfi_base_'+time_name+'.fits'

    #set fits file
    hdu = fits.PrimaryHDU(rms)
    hdulist = fits.HDUList([hdu])
    header = hdulist[0].header
    cfg_par['rfi']['chan_widths']
    #write header keywords               
    header['CRPIX1'] = 1
    header['CDELT1'] = np.abs(float(cfg_par['rfi']['chan_widths'])/1e6)
    header['CRVAL1'] = float(cfg_par['rfi']['lowfreq'])/1e6
    header['CTYPE1'] = ('Frequency')
    header['CUNIT1'] = ('MHz')
    header['CRPIX2'] = 1
    header['CRVAL2'] = 1
    header['CDELT2'] = 1
    header['CTYPE2'] =  ('Baseline')
    header['POLAR'] =  ('XX')
    header['BUNIT'] = ('% > '+str(5)+'*rms')
    header['BTYPE'] = ('intensity')
    #write file
    fits.writeto(rfi_freq_base,rms,header,overwrite=True)
    hdulist.close()

    logger.info('\t ... RFI on fits file ...\n')


    return 0


def rfi_frequency(cfg_par,time_step=-1):
    '''
    Determines the rfi per frequency channel. Saves results in table rfi_table.fits
    For each channel the flag % and factor of noise increase are stored for all, long and short baselines
    Long and short baselines are separated in half, depending on the number of baselines
    '''
    table_tmp = string.split(cfg_par['general']['msname'][0],'.MS')
    if len(table_tmp) == 1:
        table_tmp = string.split(cfg_par['general']['msname'][0],'.ms')

    if time_step != -1:
        tabledir = cfg_par['general']['timetabledir'] 
        outputdir = cfg_par['general']['rfitimedir'] 
        time_tmp = int(float(cfg_par['rfi']['chunks']['time_step'])*time_step)
        if time_tmp == 0:
            time_name = '00'+str(time_tmp)+'m'
        elif time_tmp <100:
            time_name = '0'+str(time_tmp)+'m'
        else:
            time_name= str(time_tmp)+'m'
    else:
        outputdir = cfg_par['general']['rfidir']
        tabledir = cfg_par['general']['tabledir']        
        time_name = 'full'
    
    if cfg_par['rfi']['RFInder_mode']== 'use_flags':
        rfi_freq_base =outputdir+'flags_base_'+time_name+'.fits'
        table_name = str(table_tmp[0])+'_flags_'+time_name+'.fits'
    if cfg_par['rfi']['RFInder_mode']== 'rms_clip':
        rfi_freq_base =outputdir+'rfi_base_'+time_name+'.fits'
        table_name = str(table_tmp[0])+'_rfi_'+time_name+'.fits'
    
    rfi_table = tabledir+table_name

    #open file
    if os.path.exists(rfi_freq_base) == False:
        logger.error('### Image of RFI sorted by frequency over baseline lenght does not exist ###')    
    else:    
        
        # read data and header
        hdulist = fits.open(rfi_freq_base)  # read input                
        datacube = hdulist[0].data    
        prihdr = hdulist[0].header

        #set array of frequencies
        freqs = (np.linspace(1, datacube.shape[1], datacube.shape[1])\
                     - prihdr['CRPIX1'])*prihdr['CDELT1'] + prihdr['CRVAL1']
        
        # set y-array
        rms_lin = np.zeros([datacube.shape[1]])    
        flag_lin = np.zeros([datacube.shape[1]])   

        more_long = float(cfg_par['rfi']['number_baseline'])/float(cfg_par['rfi']['num_long'])
        more_short = float(cfg_par['rfi']['number_baseline'])/float(cfg_par['rfi']['num_short'])

        rms_lin_long = np.zeros([datacube.shape[1]]) + np.sqrt(more_long)          
        rms_lin_short = np.zeros([datacube.shape[1]]) + np.sqrt(more_short)

        flag_lin_long = np.zeros([datacube.shape[1]]) + (100.-100./more_long)    
        flag_lin_short = np.zeros([datacube.shape[1]]) + (100.-100./more_short)

        natural_rms = cfg_par['rfi']['theo_rms'] 

        elevation = np.zeros(freqs.shape)+cfg_par['rfi']['altaz'].alt
        azimuth = np.zeros(freqs.shape)+cfg_par['rfi']['altaz'].az


        for i in xrange(0,datacube.shape[1]):
            
            flag_lin_tmp = np.divide(np.sum(datacube[:,i]),datacube.shape[0])
            flag_lin[i] = flag_lin_tmp


            baseline_cutoff = float(cfg_par['rfi']['baseline_cut'])
            lenghts = np.array([cfg_par['rfi']['baseline_lenghts']])+0.
            idx = (np.abs(lenghts - baseline_cutoff)).argmin()

            shortbase=datacube[:idx,i]
            longbase = datacube[idx:,i]               
            
            rms_lin_tmp = 1.-np.divide(np.divide(np.nansum(datacube[:,i]),datacube.shape[0]),100.)
            rms_lin[i] = np.divide(1.,np.sqrt(rms_lin_tmp))

            flag_lin_tmp = np.divide(np.nansum(shortbase),len(shortbase))
            flag_lin_short[i] = flag_lin_tmp
            rms_lin_tmp_short = 1.-np.divide(np.divide(np.nansum(shortbase),len(shortbase)),100.)
            rms_lin_short[i] *= np.divide(1.,np.sqrt(rms_lin_tmp_short))

            flag_lin_tmp = np.divide(np.nansum(longbase),len(longbase))
            flag_lin_long[i] = flag_lin_tmp
            rms_lin_tmp_long = 1.-np.divide(np.divide(np.nansum(longbase),len(longbase)),100.)
            rms_lin_long[i] *= np.divide(1.,np.sqrt(rms_lin_tmp_long))

        
        #rebin results
        if cfg_par['rfi']['chunks']['spw_enable'] == True:

            
            if cfg_par['rfi']['RFInder_mode']== 'use_flags':
                table_name_bin = str(table_tmp[0])+'_flags_'+time_name+'_spwbin.fits'
            if cfg_par['rfi']['RFInder_mode']== 'rms_clip':
                table_name_bin = str(table_tmp[0])+'_rfi_'+time_name+'_spwbin.fits'


            rfi_table_bin = tabledir+table_name_bin

            step_bin = cfg_par['rfi']['chunks']['spw_width']

            freqs_bin=np.arange(freqs[0],freqs[-1]+step_bin,step_bin)

            flag_lin_bin=np.zeros(freqs_bin.shape)
            rms_lin_bin=np.zeros(freqs_bin.shape)
            flag_lin_bin_short=np.zeros(freqs_bin.shape)
            rms_lin_bin_short=np.zeros(freqs_bin.shape)
            flag_lin_bin_long=np.zeros(freqs_bin.shape)
            rms_lin_bin_long=np.zeros(freqs_bin.shape)
            # is this correct ???
            natural_rms_bin=np.zeros(freqs_bin.shape)

            elevation_bin = np.zeros(freqs_bin.shape)+cfg_par['rfi']['altaz'].alt*u.deg
            azimuth_bin = np.zeros(freqs_bin.shape)+cfg_par['rfi']['altaz'].az*u.deg

            for i in xrange(0, len(freqs_bin)-1):
                #look for the right velocity bin
                index = (freqs_bin[i] <= freqs) & (freqs < freqs_bin[i+1])

                flag_lin_bin[i] = np.nanmean(flag_lin[index])
                rms_lin_bin[i] = np.nanmean(rms_lin[index])
                flag_lin_bin_short[i] = np.nanmean(flag_lin_short[index])
                rms_lin_bin_short[i] = np.nanmean(rms_lin_short[index])
                flag_lin_bin_long[i] = np.nanmean(flag_lin_long[index])
                rms_lin_bin_long[i] = np.nanmean(rms_lin_long[index])
                natural_rms_bin[i] = np.nanmean(natural_rms[index])

            # save fits table        
            c1 = fits.Column(name='frequency', format='D', unit='MHz', array=freqs_bin)
            c2 = fits.Column(name='percentage_flags', format='D', unit='-', array=flag_lin_bin)
            c3 = fits.Column(name='noise_factor', format='D', unit = '-', array=rms_lin_bin)
            c4 = fits.Column(name='percentage_flags_short', format='D', unit='-', array=flag_lin_bin_short)
            c5 = fits.Column(name='noise_factor_short', format='D', unit = '-', array=rms_lin_bin_short)
            c6 = fits.Column(name='percentage_flags_long', format='D', unit='-', array=flag_lin_bin_long)
            c7 = fits.Column(name='noise_factor_long', format='D', array=rms_lin_bin_long)        
            c8 = fits.Column(name='natural_rms', format='D', array=natural_rms_bin)        
            c9 = fits.Column(name='altitude', format='D', unit='deg', array=elevation_bin)        
            c10 = fits.Column(name='azimuth', format='D', unit='deg', array=azimuth_bin)        

            fits_table = fits.BinTableHDU.from_columns([c1, c2, c3, c4, c5, c6, c7, c8, c9, c10])    

            fits_table.writeto(rfi_table_bin, overwrite = True)


        # save fits table        
        c1 = fits.Column(name='frequency', format='D', unit='MHz', array=freqs)
        c2 = fits.Column(name='percentage_flags', format='D', unit='-', array=flag_lin)
        c3 = fits.Column(name='noise_factor', format='D', unit = '-', array=rms_lin)
        c4 = fits.Column(name='percentage_flags_short', format='D', unit='-', array=flag_lin_short)
        c5 = fits.Column(name='noise_factor_short', format='D', unit = '-', array=rms_lin_short)
        c6 = fits.Column(name='percentage_flags_long', format='D', unit='-', array=flag_lin_long)
        c7 = fits.Column(name='noise_factor_long', format='D', array=rms_lin_long)        
        c8 = fits.Column(name='natural_rms', format='D', array=natural_rms)        
        c9 = fits.Column(name='altitude', format='D', unit='deg', array=elevation)        
        c10 = fits.Column(name='azimuth', format='D', unit='deg', array=azimuth)        
        
        fits_table = fits.BinTableHDU.from_columns([c1, c2, c3, c4, c5, c6, c7, c8])    
        
        fits_table.writeto(rfi_table, overwrite = True)
      
        logger.info("\t ... RFI table saved ...\n")

        return 0

# Content to be published
def write_html_fullreport(cfg_par):

    # Configure Jinja and ready the template
    env = Environment(
        loader=FileSystemLoader(cfg_par['general']['template_folder'])
    )

    #base_template = env.get_template('report.html')
    # Content to be published
    title = 'RFI report: {0:s}'.format(cfg_par['general']['msname'])

    #imagename1 = '/Users/maccagni/Projects/RFI/rfinder_test/rfi/plots/altaz/AltAZ_rfi1297-1317MHz.png'
    #data_uri1 = open(imagename1, 'rb').read().encode('base64').replace('\n', '')
    
    if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
        imagename1 = cfg_par['general']['plotdir']+'rfi_base_full.png'
        imagename3 = cfg_par['general']['plotdir']+'noise_full_sl_rfi.png'
    elif cfg_par['rfi']['RFInder_mode'] == 'use_flags':
        imagename1 = cfg_par['general']['plotdir']+'flags_base_full.png'
        imagename3 = cfg_par['general']['plotdir']+'noise_full_sl_flags.png'
    
    data_uri1 = open(imagename1, 'rb').read().encode('base64').replace('\n', '')

    imagename2 = cfg_par['general']['plotdir']+'AltAZ_full.png'
    data_uri2 = open(imagename2, 'rb').read().encode('base64').replace('\n', '')

    data_uri3 = open(imagename3, 'rb').read().encode('base64').replace('\n', '')

    video_name1 = cfg_par['general']['moviedir']+'AltAz_movie.gif'
    if os.path.exists(video_name1):
        video_encoded1 = open(video_name1, "rb").read().encode("base64")

    video_name2 = cfg_par['general']['moviedir']+'Time_2Dplot_movie.gif'
    if os.path.exists(video_name2):
        video_encoded2 = open(video_name2, "rb").read().encode("base64")

    video_name3 = cfg_par['general']['moviedir']+'TimeChunks_1D_noise.gif'
    if os.path.exists(video_name3):
        video_encoded3 = open(video_name3, "rb").read().encode("base64")

    if cfg_par['plots']['movies']['movies_in_report'] == True:
        template = env.get_template('full_template.html')
        with open(cfg_par['general']['rfidir']+'full_report.html', "w") as f:
            lenghts = np.array([cfg_par['rfi']['baseline_lenghts']])+0.
            f.write(template.render(
                title=title,
                fieldname=cfg_par['general']['fieldname'],
                field=cfg_par['general']['field'],
                totchans = int(cfg_par['rfi']['total_channels']),
                chan_widths=round(cfg_par['rfi']['chan_widths']/1e3,4),
                lowfreq=round(cfg_par['rfi']['lowfreq']/1e6,3),
                highfreq=round(cfg_par['rfi']['highfreq']/1e6,3),
                startdate = ('{0:%y}{0:%b}{0:%d} {0:%X}'.format(cfg_par['rfi']['startdate'].datetime)),
                enddate =   ('{0:%y}{0:%b}{0:%d} {0:%X}'.format(cfg_par['rfi']['enddate'].datetime)),
                nant = cfg_par['rfi']['nant'],
                ant_names = cfg_par['rfi']['ant_names'],
                maxbase = str(np.round(lenghts[0][-1],0)),
                minbase = str(np.round(lenghts[0][0],0)),
                totbase = cfg_par['rfi']['number_baseline'],
                exptime = np.round(cfg_par['rfi']['exptime'],2),
                polnum = cfg_par['rfi']['polnum'],
                noise = np.round(cfg_par['rfi']['theo_rms'][0]*1e3,5),
                img_tag1 = '<img class="a" src="data:image/png;base64,{0}">'.format(data_uri1),
                video_tag1 = '<img class="b" src="data:video/gif;base64,{0}">'.format(video_encoded1),
                img_tag3 = '<img class="c" src="data:image/png;base64,{0}">'.format(data_uri3),
                #video_tag1 = '<img class="d" src="data:video/gif;base64,{0}">'.format(video_encoded1),        
                video_tag2 = '<img class="e" src="data:video/gif;base64,{0}">'.format(video_encoded2),        
                video_tag3 = '<img class="f" src="data:video/gif;base64,{0}">'.format(video_encoded3)                    
            ))

    elif cfg_par['plots']['movies']['movies_in_report'] == False:
        template = env.get_template('fullshort_template.html')
        with open(cfg_par['general']['rfidir']+'full_report.html', "w") as f:
            lenghts = np.array([cfg_par['rfi']['baseline_lenghts']])+0.
            f.write(template.render(
                title=title,
                fieldname=cfg_par['general']['fieldname'],
                field=cfg_par['general']['field'],
                totchans = int(cfg_par['rfi']['total_channels']),
                chan_widths=round(cfg_par['rfi']['chan_widths']/1e3,4),
                lowfreq=round(cfg_par['rfi']['lowfreq']/1e6,3),
                highfreq=round(cfg_par['rfi']['highfreq']/1e6,3),
                startdate = ('{0:%y}{0:%b}{0:%d} {0:%X}'.format(cfg_par['rfi']['startdate'].datetime)),
                enddate =   ('{0:%y}{0:%b}{0:%d} {0:%X}'.format(cfg_par['rfi']['enddate'].datetime)),
                nant = cfg_par['rfi']['nant'],
                ant_names = cfg_par['rfi']['ant_names'],
                maxbase = str(np.round(lenghts[0][-1],0)),
                minbase = str(np.round(lenghts[0][0],0)),
                totbase = cfg_par['rfi']['number_baseline'],
                exptime = np.round(cfg_par['rfi']['exptime'],2),
                polnum = cfg_par['rfi']['polnum'],
                noise = np.round(cfg_par['rfi']['theo_rms'][0]*1e3,5),
                img_tag1 = '<img class="a" src="data:image/png;base64,{0}">'.format(data_uri1),
                video_tag1 = '<img class="b" src="data:video/gif;base64,{0}">'.format(data_uri2),
                img_tag3 = '<img class="c" src="data:image/png;base64,{0}">'.format(data_uri3),                 
            ))

        logger.info('---- Html report done ----\n')

        return 0

def write_html_timereport(cfg_par):

    # Configure Jinja and ready the template
    env = Environment(
        loader=FileSystemLoader(cfg_par['general']['template_folder'])
    )
    template = env.get_template('time_template.html')

    #base_template = env.get_template('report.html')
    # Content to be published
    title = 'RFI time scans report: {0:s}'.format(cfg_par['general']['msname'])

    #imagename1 = '/Users/maccagni/Projects/RFI/rfinder_test/rfi/plots/altaz/AltAZ_rfi1297-1317MHz.png'
    #data_uri1 = open(imagename1, 'rb').read().encode('base64').replace('\n', '')
    video_name1 = cfg_par['general']['moviedir']+'AltAz_movie.gif'
    if os.path.exists(video_name1):
        video_encoded1 = open(video_name1, "rb").read().encode("base64")
    else:
        video_encoded1 = None
    
    video_name2 = cfg_par['general']['moviedir']+'Time_2Dplot_movie.gif'
    if os.path.exists(video_name2):
        video_encoded2 = open(video_name2, "rb").read().encode("base64")
    else:
        video_encoded1 = None

    video_name3 = cfg_par['general']['moviedir']+'TimeChunks_1D_noise.gif'
    if os.path.exists(video_name3):
        video_encoded3 = open(video_name3, "rb").read().encode("base64")
    else:
        video_encoded3 = None

    if cfg_par['plots']['movies']['movies_in_report'] == True:

        with open(cfg_par['general']['rfidir']+'time_report.html', "w") as f:
            lenghts = np.array([cfg_par['rfi']['baseline_lenghts']])+0.
            f.write(template.render(
                title=title,
                fieldname=cfg_par['general']['fieldname'],
                field=cfg_par['general']['field'],
                totchans = int(cfg_par['rfi']['total_channels']),
                chan_widths=round(cfg_par['rfi']['chan_widths']/1e3,4),
                lowfreq=round(cfg_par['rfi']['lowfreq']/1e6,3),
                highfreq=round(cfg_par['rfi']['highfreq']/1e6,3),
                startdate = ('{0:%y}{0:%b}{0:%d} {0:%X}'.format(cfg_par['rfi']['startdate'].datetime)),
                enddate =   ('{0:%y}{0:%b}{0:%d} {0:%X}'.format(cfg_par['rfi']['enddate'].datetime)),
                nant = cfg_par['rfi']['nant'],
                ant_names = cfg_par['rfi']['ant_names'],
                maxbase = str(np.round(lenghts[0][-1],0)),
                minbase = str(np.round(lenghts[0][0],0)),
                totbase = cfg_par['rfi']['number_baseline'],
                exptime = np.round(cfg_par['rfi']['exptime']*60.,2),
                polnum = cfg_par['rfi']['polnum'],
                noise = np.round(cfg_par['rfi']['theo_rms'][0]*1e3,5),
                video_tag1 = '<img class="b" src="data:video/gif;base64,{0}">'.format(video_encoded1),        
                video_tag2 = '<img class="a" src="data:video/gif;base64,{0}">'.format(video_encoded2),        
                video_tag3 = '<img class="c" src="data:video/gif;base64,{0}">'.format(video_encoded3)                    
            ))

    elif cfg_par['plots']['movies']['movies_in_report'] == False:

        logger.info('\t ERROR:  movies in report must be set to TRUE\n')

        return 0

    logger.info('---- Html time report done ----\n')

    return 0

def find_altaz_plots(cfg_par):
    
    tmp_arr=[]

    if cfg_par['rfi']['RFInder_mode']=='use_flags':
        filenames = glob.glob(cfg_par['general']['altazplotdir']+'/AltAZ_flags*')

        for i in xrange(0,len(filenames)):
            tmp = filenames[i].split('_flags')
            tmp_arr.append(tmp[1].split('MHz')[0])
        tmp_arr.sort()
            
        filenames = [tmp[0]+'_flags' + s for s in tmp_arr] 
        filenames = [s + 'MHz.png' for s in filenames] 
    
    elif cfg_par['rfi']['RFInder_mode']=='rms_clip':
        filenames = glob.glob(cfg_par['general']['altazplotdir']+'/AltAZ_rfi*')

        for i in xrange(0,len(filenames)):
            tmp = filenames[i].split('_rfi')
            tmp_arr.append(tmp[1].split('MHz')[0])
        tmp_arr.sort()
            
        filenames = [tmp[0]+'_rfi' + s for s in tmp_arr] 
        filenames = [s + 'MHz.png' for s in filenames] 

    return filenames

def find_2d_plots(cfg_par):

    if cfg_par['rfi']['RFInder_mode']=='use_flags':
        filenames = glob.glob(cfg_par['general']['timeplotdir2D']+'/flags_base*')
    elif cfg_par['rfi']['RFInder_mode']=='rms_clip':
        filenames = glob.glob(cfg_par['general']['timeplotdir2D']+'/rfi_base*')

    tmp_arr=[]
    for i in xrange(0,len(filenames)):
            tmp = filenames[i].split('base_')[1]
            tmp_arr.append(tmp.split('m.png')[0])
    tmp_arr.sort()

    tmp = filenames[0].split('base_')
    filenames = [tmp[0]+'base_' + s for s in tmp_arr] 
    filenames = [s + 'm.png' for s in filenames] 

    return filenames

def find_1d_plots(cfg_par,name_root):

    #select files
    filenames = glob.glob(cfg_par['general']['timeplotdir1D']+'/'+name_root+'_*')
    #print filenames 

    tmp_arr=[]

    for i in xrange(0,len(filenames)):
        if cfg_par['rfi']['RFInder_mode']=='use_flags':
            if len(filenames[i].split('sl_rfi.'))>1:
                continue
        elif cfg_par['rfi']['RFInder_mode']=='rms_clip':
            if  len(filenames[i].split('sl_flags.'))>1:
                continue
        tmp = filenames[i].split(name_root+'_')
        tmp_arr.append(tmp[1].split('m_sl_rfi.png')[0])

    tmp_arr.sort()
    tmp = filenames[0].split(name_root+'_')

    filenames = [tmp[0]+name_root+'_' + s for s in tmp_arr]
    #if cfg_par['rfi']['RFInder_mode']=='use_flags':
    #    filenames = [s + 'm_sl_flags.png' for s in filenames] 
    #elif cfg_par['rfi']['RFInder_mode']=='rms_clip':
    #    filenames = [s + 'm_sl_rfi.png' for s in filenames] 
    
    #print filenames

    return filenames

def set_dirs(cfg_par):
    '''
 
    Sets temporary subtirectories
    Creates directory rfi_stokes/
    Subdirectories can be kept using no_cleanup option

    '''

    key = 'general'

    if os.path.exists(cfg_par[key]['moviedir']) == False:
         os.makedirs(cfg_par[key]['moviedir'])

    if os.path.exists(cfg_par[key]['rfidir']) == False:
         os.makedirs(cfg_par[key]['rfidir'])           

    if os.path.exists(cfg_par[key]['tabledir']) == False:
         os.makedirs(cfg_par[key]['tabledir'])

    if os.path.exists(cfg_par[key]['plotdir']) == False:
         os.makedirs(cfg_par[key]['plotdir'])

    if cfg_par['rfi']['chunks']['time_enable'] == True:

        if os.path.exists(cfg_par[key]['rfitimedir']) == False:
             os.makedirs(cfg_par[key]['rfitimedir'])

        if os.path.exists(cfg_par[key]['timetabledir']) == False:
             os.makedirs(cfg_par[key]['timetabledir'])

        if os.path.exists(cfg_par[key]['timechunksdir']) == False:
             os.makedirs(cfg_par[key]['timechunksdir'])

        if os.path.exists(cfg_par[key]['timeplotdir1D']) == False:
             os.makedirs(cfg_par[key]['timeplotdir1D'])

        if os.path.exists(cfg_par[key]['timeplotdir2D']) == False:
             os.makedirs(cfg_par[key]['timeplotdir2D'])

        if os.path.exists(cfg_par[key]['altazplotdir']) == False:
             os.makedirs(cfg_par[key]['altazplotdir'])

        return(0)

def cleanup(cfg_par):

    key = 'general'


    if cfg_par['rfi']['chunks']['time_enable'] == True:
        if os.path.exists(cfg_par[key]['timeplotdir1D']):
            shutil.rmtree(cfg_par[key]['timeplotdir1D'])

        if os.path.exists(cfg_par[key]['timeplotdir2D']):
            shutil.rmtree(cfg_par[key]['timeplotdir2D']) 

        if os.path.exists(cfg_par[key]['altazplotdir']):
            shutil.rmtree(cfg_par[key]['altazplotdir']) 

        if os.path.exists(cfg_par[key]['timechunksdir']):
            shutil.rmtree(cfg_par[key]['timechunksdir']) 

        if os.path.exists(cfg_par[key]['tabledir']):
            shutil.rmtree(cfg_par[key]['tabledir'])

        if os.path.exists(cfg_par[key]['rfitimedir']):
            shutil.rmtree(cfg_par[key]['rfitimedir'])

    else:
        if os.path.exists(cfg_par[key]['plotdir']):
            shutil.rmtree(cfg_par[key]['plotdir'])
        if os.path.exists(cfg_par[key]['tabledir']):
            shutil.rmtree(cfg_par[key]['tabledir'])
            
    fitsname = glob.glob(cfg_par[key]['rfidir']+'*.fits')

    if len(fitsname)>0:
        for i in xrange(0,len(fitsname)):
            os.remove(fitsname[i])




# Import modules
import os
import sys
import string
import numpy as np
import yaml
import json
import glob
import argparse
from  argparse import ArgumentParser
import textwrap as _textwrap
import logging
import logging.config

import warnings
import pkg_resources

from astropy.io import fits, ascii
from astropy import units as u
from astropy.time import Time, TimeDelta
from astropy.table import Table, Column, MaskedColumn



# get rfinder install directory
RFINDER_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
RFINDER_DIR = RFINDER_PATH+'/rfinder/'

sys.path.append(os.path.join(RFINDER_PATH, 'rfinder'))

import rfi
import rfinder_stats as rfi_stats
import rfinder_plots as rfi_plots
import rfinder_files as rfiFL

rfi = rfi.rfi()
rfiST = rfi_stats.rfi_stats()
rfiPL = rfi_plots.rfi_plots()

DEFAULT_CONFIG = 'rfinder_default.yml'
# Set up logging infrastructure
LOG_FILE = 'log-rfinder.log'
if os.path.exists(LOG_FILE) == True:
    os.remove(LOG_FILE)

# This is is the default log file. It logs stimela images, containers and processes

if not sys.warnoptions:
    warnings.simplefilter("ignore")


try:
    __version__ = pkg_resources.require("rfinder")[0].version
except pkg_resources.DistributionNotFound:
    __version__ = "dev"

####################################################################################################

class MultilineFormatter(argparse.HelpFormatter):
    def _fill_text(self, text, width, indent):
        text = self._whitespace_matcher.sub(' ', text).strip()
        paragraphs = text.split('|n ')
        multiline_text = ''
        for paragraph in paragraphs:
            formatted_paragraph = _textwrap.fill(paragraph, width, initial_indent=indent, subsequent_indent=indent) + '\n\n'
            multiline_text = multiline_text + formatted_paragraph
        return multiline_text


def is_valid_file(parser, arg):
    if not os.path.exists(arg):
        parser.error("The file '%s' does not exist!" % arg)

    return arg



class rfinder:
    '''

    Class to investigate the RFI behaviour during observations

    '''

    def __init__(self):
        '''

        Set self.logger for spectrum extraction
        Find config file
        If not specified by user load rfinder_default.yml

        '''

        # set self.logger
        #with open(RFINDER_DIR+'/templates/logcfg.yml', 'r') as f:
        #    config = yaml.safe_load(f.read())
        #    logging.config.dictConfig(config)

        self.logger = logging.getLogger('log-rfinder.log')
        #self.logger.setLevel(logging.INFO)

        #fh = logging.FileHandler('log-rfinder.log')
        #fh.setLevel(logging.INFO)

        #ch = logging.StreamHandler()
        #ch.setLevel(logging.WARNING)

        #formatter = logging.Formatter('%(levelname)s - %(filename)s - %(message)s')
        #fh.setFormatter(formatter)
        #ch.setFormatter(formatter)

        #self.logger.addHandler(ch)
        #self.logger.addHandler(fh)


    def readArgs(self,argv):


        self.parser = ArgumentParser(description='RFInder: package to visualize the flagged RFI in a dataset '
                                '|n version {:s} |n install path {:s} |n '
                                'Filippo Maccagni <filippo.maccagni@gmial.com>'.format(__version__,
                                                                                   os.path.dirname(__file__)),
                                formatter_class=MultilineFormatter,
                                add_help=False)

        add = self.parser.add_argument

        add("-h", "--help",  action="store_true",
                help="Print help message and exit")

        add("-v","--version", action='version',
                version='{:s} version {:s}'.format(self.parser.prog, __version__))

        add('-c', '--config',
            type=lambda a: is_valid_file(self.parser, a),
            default=False,
            help='RFInder configuration file (YAML format)')

        add('-idir', '--input_dir',
            type= str,
            default = False,
            help= 'select working directory (MS file assumed to be here)')

        add('-odir', '--output_dir',
            type=str,
            default=False,
            help='select output directory')

        add('-l','--label',
            type=str,
            default=False,
            help='ouput folder is called: rfi_polarization_label')

        add('-i', '--input',
            type=str,
            default=False,
            help='''input ['MS'] file''')

        add('-fl', '--field',
            type=int,
            default=False,
            help='select field of MS file to analyze')

        add('-tel', '--telescope',
            type=str,
            default=False,
            help='select telescope: meerkat, apertif, wsrt')

        add('-mode', '--rfimode',
            type=str,
            default=False,
            help='select mode where to investigate RFI: use_flags or rms_clip')

        add('-pol', '--polarization',
            type=str,
            default=False,
            help='select stokes parameter: xx, yy, xy, yx, q (also in CAPS)')

        add('-fint', '--frequency_interval',
            nargs='*',
            default=False,
            help='select frequency interval where to measure noise in GHz')

        add('-spwAv', '--spw_av',
            type=int,
            default=False,
            help='select number of channels to average')

        add('-tStep', '--time_step',
            type=int,
            default=False,
            help='select time step in minutes in which divide the analysis of the MSfile')

        add('-sig', '--sigma_clip',
            type=int,
            default=False,
            help='select sigma clip for rms_clip mode to find RFI')

        add('-baseCut', '--baseline_cut',
            type=int,
            default=False,
            help='select cut in baseline lenght [m] for differential RFI analysis')

        add('-noCh', '--no_chunks',
            action='store_true',
            help='desable chunking in time')

        add('-yesCh','--yes_chunks',
            action='store_true',
            help='enable chunking in time')

        add('-noMov', '--no_movies',
            action='store_true',
            help='disable movies (use if dataset is read as a whole)')

        add('-noSpw', '--no_spw_av',
            action='store_true',
            help='disable averaging in channels')

        add('-yesSpw','--yes_spw_av',
            action='store_true',
            help='enable averaging in channels')

        add('-noClp','--no_cleanup',
            action='store_true',
            help='disable cleanup of intermediate products')

        add('-yesClp','--yes_cleanup',
            action='store_true',
            help='enable cleanup of intermediate products')



        args = self.parser.parse_args(argv)
        
        return args


    def setArgs(self,args):

        if args.input_dir:
            self.cfg_par['general']['workdir'] = args.input_dir
        if args.output_dir:
            self.cfg_par['general']['outdir'] = args.output_dir
        if args.input:
            self.cfg_par['general']['msname'] = args.input
        if args.field != None:
            self.cfg_par['general']['field'] = args.field
        if args.telescope:
            self.cfg_par['general']['telescope'] = args.telescope
        if args.polarization:
            self.cfg_par['rfi']['polarization'] = args.polarization
        if args.baseline_cut:
            self.cfg_par['rfi']['baseline_cut'] = args.baseline_cut
        if args.time_step:
            self.cfg_par['rfi']['chunks']['time_enable'] = True
            self.cfg_par['rfi']['chunks']['time_step'] = args.time_step
        if args.spw_av:
            self.cfg_par['rfi']['chunks']['spw_enable'] = True
            self.cfg_par['rfi']['chunks']['spw_width'] = args.spw_av
 
        if args.no_chunks==True:
                self.cfg_par['rfi']['chunks']['time_enable'] = False
        if args.yes_chunks == True:
                self.cfg_par['rfi']['chunks']['time_enable'] = True

        if args.no_spw_av==True:
            self.cfg_par['rfi']['chunks']['spw_enable'] = False
        
        if args.yes_spw_av==True:
            self.cfg_par['rfi']['chunks']['spw_enable'] = True

        if args.no_movies == True : 
            self.cfg_par['plots']['movies']['movies_in_report'] = False

        if args.no_cleanup==True:
            self.cfg_par['general']['cleanup_enable'] = False
        if args.yes_cleanup==True:
            self.cfg_par['general']['cleanup_enable'] = True
        
        if args.label !=False :
            self.cfg_par['general']['outlabel'] = '_'+args.label
        
        else:
            self.cfg_par['general']['outlabel'] = ''               
        
        if (args.rfimode == 'rms_clip' or args.rfimode == 'rms') :
                self.cfg_par['rfi']['RFInder_mode'] = args.rfimode
                if args.sigma_clip:
                    self.cfg_par['rfi']['rms_clip'] = args.sigma_clip
                if args.frequency_interval:
                    self.cfg_par['rfi']['noise_measure_edges'] = args.frequency_interval

        return self

    def set_cfg_par(self):

        key = 'general'

        workdir  = self.cfg_par[key].get('workdir', None)
        msfile = workdir + self.cfg_par[key].get('msname', None)
        self.cfg_par[key]['msfullpath'] = msfile      

        outdir  = self.cfg_par[key].get('outdir', None)
        rfidir  = outdir+'rfi_'+self.cfg_par['rfi']['polarization']+self.cfg_par[key]['outlabel']+'/'
        self.cfg_par[key]['rfidir'] = rfidir

        tabledir = rfidir+'tables/'
        self.cfg_par[key]['tabledir'] = tabledir
    
        rfiplotdir = rfidir+'plots/'
        self.cfg_par[key]['plotdir'] = rfiplotdir 

        moviedir = rfidir+'plots/movies/'
        self.cfg_par[key]['moviedir'] = moviedir             

        rfitimedir = rfidir+'time_chunks/'
        self.cfg_par[key]['rfitimedir'] = rfitimedir

        timetabledir = tabledir+'time_chunks/'
        self.cfg_par[key]['timetabledir'] = timetabledir

        timeplotdir_tmp = rfiplotdir+'time_chunks/'
        self.cfg_par[key]['timechunksdir'] = timeplotdir_tmp          

        timeplotdir1d = timeplotdir_tmp+'1D/'
        self.cfg_par[key]['timeplotdir1D'] = timeplotdir1d

        timeplotdir2d = timeplotdir_tmp+'2D/'
        self.cfg_par[key]['timeplotdir2D'] = timeplotdir2d

        altazplotdir = rfidir+'plots/altaz/'
        self.cfg_par[key]['altazplotdir'] = altazplotdir        


        return 0




    def go(self,cfg_par):
        '''
        Automated pipeline to extract spectra from each continuum source in a given field.
        If cfg_par['rfi'] is enabled 
            Executes the whole spectrum extraction process as follows:
            1: load_from_ms
            2: baselines_from_ms 
            3: priors_flag
            4: rfi_flag
        If cfg_par['plots'] is enabled
            1: 2d plot of RFI flagged by frequency and baseline lenght (plot_rfi_im)
            2: 1d plot of RFI flagged by frequency channel (baselines_from_ms)
            3: 1d plot of noise increase by frequency channel (for long and short baselines) (priors_flag)
        If cfg_par['rfi']['chunks']['time_chunks'] is enabled
            1: executes 'rfi' and 'plots' procedure dividing the observation in time-steps given by cfg_par['rfi']['chunks']['time_step']
            2: collects the info about the % of RFI for each time step in Alt/Az plots 
        If cfg_par['beam_shape'] is enabled
            1: create FLAG column in MS file (rfi_flag)
            2: determine psf using wsclean (make_psf)
        '''

        # cont_sources

        self.cfg_par = cfg_par
        task = 'rfi'
        rfiFL.set_dirs(self.cfg_par)

        self.logger.warning("------ STARTING RFI analysis ------\n")

        if self.cfg_par[task]['rfi_enable']==True:
            
            if self.cfg_par[task]['chunks']['time_enable']==True:

                times, start, end = rfiST.time_chunk(self.cfg_par)
                self.logger.warning("------ Working on time chunks ------\n")

                for i in xrange(0,len(times)-1):
                    timez = [times[i],times[i+1]] 
                    
                    #time chunk properties
                    time_delta = float(self.cfg_par['rfi']['chunks']['time_step'])*i
                    time_del = TimeDelta(time_delta*60., format='sec')
                    time_delta_plus = TimeDelta(float(self.cfg_par['rfi']['chunks']['time_step'])*60., format='sec')
                    start = self.cfg_par['rfi']['startdate']+time_del
                    end = start+time_delta_plus
                    self.logger.warning((" ------ Working on chunk #{0:d}:").format(i))
                    self.logger.warning(("\tbetween {0:%d}{0:%b}{0:%y}: {0:%H}:{0:%M} - {1:%H}:{1:%M}\n").format(start.datetime,end.datetime))

                    result = rfi.load_from_ms(self.cfg_par,timez,i)
                    self.logger.info("------ MSfile Loaded ------\n")

                    #sort visibilities by baseline lenght
                    if result != 1:
                        rfi.baselines_from_ms(self.cfg_par)
                        self.logger.info("------ Dataset sorted by baseline lenght ------\n")

                        #flag bad antennas (from configuration file)
                        datas = rfi.priors_flag(self.cfg_par)
                        self.logger.info("------ Bad antennas and autocorrelations flagged ------\n")

                        #find rfi above threshold
                        rfi.find_rfi(datas,self.cfg_par,i)
                        self.logger.info(" ------  RFI found  ------\n")

                        rfiFL.rfi_frequency(self.cfg_par,i)
                        self.logger.info("------ RFI saved to table ------\n")
                    else:
                        self.logger.warning(" ------ This chunk is empty ------\n")
                        continue

                self.logger.info("------ End of RFI analysis on time chunks ------\n")

            else:

                rfi.load_from_ms(self.cfg_par,0,0)
                #determine alt/az

                self.logger.warning("------ MSfile Loaded -----\n")
                rfi.baselines_from_ms(self.cfg_par)
                self.logger.warning("------ Dataset sorted by baseline lenght ------\n")
                datas = rfi.priors_flag(self.cfg_par)
                self.logger.warning("------ Bad antennas and autocorrelations flagged ------\n")
                rfi.find_rfi(datas,self.cfg_par,-1)
                self.logger.warning("------  RFI found  ------\n")
                rfiFL.rfi_frequency(self.cfg_par,-1)
                self.logger.warning("------ RFI saved to table ------\n")
                self.logger.warning("------ End of RFI analysis ------\n")
      
        task = 'plots'
        if self.cfg_par[task]['plot_enable']==True:
            
            if self.cfg_par['rfi']['chunks']['time_enable']==True:

                times, start, end = rfiST.time_chunk(self.cfg_par)
                self.logger.warning(" ------ Plotting on time chunks ------\n")

                for i in xrange(0,len(times)-1):
#                for i in xrange(0,2):

                    timez = [times[i],times[i+1]]            
                    
                    #time chunk properties
                    time_delta = float(self.cfg_par['rfi']['chunks']['time_step'])*i
                    time_del = TimeDelta(time_delta*60., format='sec')
                    time_delta_plus = TimeDelta(float(self.cfg_par['rfi']['chunks']['time_step'])*60., format='sec')
                    start = self.cfg_par['rfi']['startdate']+time_del
                    end = start+time_delta_plus
                    
                    self.logger.info((" ------ Plotting chunk #{0:d}:").format(i))
                    self.logger.info(("\t \t between {0:%d}{0:%b}{0:%y}: {0:%H}:{0:%M} - {1:%H}:{1:%M}").format(start.datetime,end.datetime))

                    if self.cfg_par['rfi']['rfi_enable']==False:

                        results = rfi.load_from_ms(self.cfg_par,timez,i)
                        self.logger.info("------ MSfile Loaded ------\n")    
                        if results != 1:
                            rfi.baselines_from_ms(self.cfg_par)
                            self.logger.info("------ Dataset sorted by baseline lenght ------\n")
                        else:
                            self.logger.warning("------ This chunk is empty ------\n")
                            continue       

                    rfiPL.plot_rfi_imshow(self.cfg_par,i)
                    self.logger.info("------ RFI in 2D plotted ------\n")
                    self.cfg_par['plots']['plot_noise'] = 'rfi'
                    self.cfg_par['plots']['long_short'] = False
                    rfiPL.plot_noise_frequency(self.cfg_par,i)
                    self.cfg_par['plots']['long_short'] = True
                    rfiPL.plot_noise_frequency(self.cfg_par,i)
                    self.cfg_par['plots']['plot_noise'] = 'noise_factor'
                    rfiPL.plot_noise_frequency(self.cfg_par,i)
                    self.cfg_par['plots']['plot_noise'] = 'noise'
                    rfiPL.plot_noise_frequency(self.cfg_par,i)         
                    self.logger.warning("------ RFI in 1D plotted ------\n")
                
                rfiPL.plot_altaz(self.cfg_par,68)
                self.logger.warning("------ RFI in ALT/AZ plotted ------\n")
        
                if (self.cfg_par['plots']['movies']['altaz_gif']==True or self.cfg_par['plots']['movies']['2d_gif']==True or 
                    self.cfg_par['plots']['movies']['1d_gif']==True):
                    self.logger.warning("------ Making movies ------\n")
     
                if self.cfg_par['plots']['movies']['altaz_gif']==True:

                    out_animation = self.cfg_par['general']['moviedir']+'AltAz_movie.gif'
                    filenames = rfiFL.find_altaz_plots(self.cfg_par)
                    rfiPL.gif_me_up(self.cfg_par,filenames,out_animation)
                    self.logger.info("------ AltAz movie done ------\n")
                
                if self.cfg_par['plots']['movies']['2d_gif']==True:
                    out_animation = self.cfg_par['general']['moviedir']+'Time_2Dplot_movie.gif'
                    filenames = rfiFL.find_2d_plots(self.cfg_par)
                    rfiPL.gif_me_up(self.cfg_par,filenames,out_animation)
                
                    self.logger.info("------ 2D movie done ------\n")
                
                if self.cfg_par['plots']['movies']['1d_gif']==True:
                    out_animation = self.cfg_par['general']['moviedir']+'TimeChunks_1D_flags.gif'
                    root_name = 'flags'
                    filenames = rfiFL.find_1d_plots(self.cfg_par,root_name)
                    rfiPL.gif_me_up(self.cfg_par,filenames,out_animation)
                    
                    out_animation = self.cfg_par['general']['moviedir']+'TimeChunks_1D_noise.gif'
                    root_name = 'noise'
                    filenames = rfiFL.find_1d_plots(self.cfg_par,root_name)
                    rfiPL.gif_me_up(self.cfg_par,filenames,out_animation)

                    out_animation = self.cfg_par['general']['moviedir']+'TimeChunks_1D_noisefactor.gif'
                    root_name = 'noisefactor'
                    filenames = rfiFL.find_1d_plots(self.cfg_par,root_name)
                    rfiPL.gif_me_up(self.cfg_par,filenames,out_animation)         
                    self.logger.info("------ 1D movies done ------\n")
                
                if (self.cfg_par['plots']['movies']['altaz_gif']==True or self.cfg_par['plots']['movies']['2d_gif']==True or 
                    self.cfg_par['plots']['movies']['1d_gif']==True):
                    self.logger.warning("------ Movies done ------\n")

                rfiFL.write_html_timereport(self.cfg_par)                 

            else:
                
                if self.cfg_par['rfi']['rfi_enable']==False:

                    results = rfi.load_from_ms(self.cfg_par,0,0)
                    self.logger.info("------ MSfile Loaded -----\n")    
                    if results != 1:
                        rfi.baselines_from_ms(self.cfg_par)
                        self.logger.info("------ Dataset sorted by baseline lenght ------\n")
                    else:
                        self.logger.info("------ This dataset is empyy ------\n")

                rfiPL.plot_altaz_short(self.cfg_par)
                self.logger.warning("------ Alt/Az plotted ------\n")                            
                rfiPL.plot_rfi_imshow(self.cfg_par,-1)
                self.logger.warning("------ RFI in 2D plotted ------\n")
                self.cfg_par['plots']['plot_noise'] = 'rfi'
                self.cfg_par['plots']['long_short'] = False
                rfiPL.plot_noise_frequency(self.cfg_par,-1)
                self.cfg_par['plots']['long_short'] = True
                rfiPL.plot_noise_frequency(self.cfg_par,-1)
                self.cfg_par['plots']['plot_noise'] = 'noise_factor'
                rfiPL.plot_noise_frequency(self.cfg_par,-1)
                self.cfg_par['plots']['plot_noise'] = 'noise'
                rfiPL.plot_noise_frequency(self.cfg_par,-1)
                self.logger.warning("------ RFI in 1D plotted ------\n")

                rfiFL.write_html_fullreport(self.cfg_par)

        self.logger.info("------ cleaning up ------\n")
        
        if self.cfg_par['general']['cleanup_enable'] == True:
            rfiFL.cleanup(self.cfg_par)

        self.logger.warning("------ End of RFInder ------\n\n")

        return 0



    def main (self,argv):
        
        for i, arg in enumerate(argv):
            if (arg[0] == '-') and arg[1].isdigit(): argv[i] = ' ' + arg

        args = self.readArgs(argv)

        if args.help:  #rfinder -h 
            self.parser.print_help()

            print("""\nRun a command. This can be:\n \nrfinder \nrfinder -c path_to_config_file.yml
rfinder -i <ngc1399.ms> -fl <num> -tel <meerkat/apertif/wsrt>\n""")

            sys.exit(0)

        elif args.config:    #rfinder -c config_file.yml         
            self.logger.warning('------ Reading your parameter file ------\n')
            # read database here
            files =  args.config
            cfg = open(files)
            self.cfg_par = yaml.load(cfg)

        else: #rfinder  or rfinder -options
            workdir = os.getcwd()
            workdir = workdir+'/'
            exists = os.path.isfile(workdir+'/'+DEFAULT_CONFIG)
            if exists:
                self.logger.warning('------ Reading default parameter file in your directory ------\n')
                file_default = os.path.join(workdir, DEFAULT_CONFIG)
                cfg = open(file_default)
                self.cfg_par = yaml.load(cfg)
            else:
                # Keep presets
                self.logger.warning('------ Reading default installation parameter file ------\n')
                file_default = os.path.join(RFINDER_DIR, DEFAULT_CONFIG)
                cfg = open(file_default)
                self.cfg_par = yaml.load(cfg)            
                self.cfg_par['general']['workdir'] = workdir
                self.cfg_par['general']['outdir'] = workdir
                with open(workdir+DEFAULT_CONFIG, 'w') as outfile:
                    yaml.dump(self.cfg_par, outfile, default_flow_style=False)

                if args.field ==False and args.input==False :

                    self.logger.warning('''MSNAME & telescope missing
              \t\tplease edit rfinder_default.yml in your current directory
              \t\tor run: rfinder -i msname -fl <field_number> -tel <meerkat,apertif,wsrt>
              \t\t(assuming the observation is located in your current directory)
                    \n''')
                    self.logger.critical('''------ RFInder out ------\n''')
                    
                    sys.exit(0)

                else:
                    self.logger.warning('''------ you provided MSname and telescope in your first run, 
            \tassuming MS is your current directory ------\n''')
 
            if all(x==False for x in vars(args).values()) == False and (args.help==False and args.config == False):
                self.logger.warning('------ Updating arguments given from terminal ------\n')

                self.setArgs(args)
                with open(workdir+DEFAULT_CONFIG, 'w') as outfile:
                    yaml.dump(self.cfg_par, outfile, default_flow_style=False)
 
        self.cfg_par['general']['template_folder'] = os.path.join(RFINDER_PATH,'rfinder/templates')
        self.set_cfg_par()

        return self



import matplotlib
matplotlib.use('agg')
import os,string,sys, glob
import numpy as np

matplotlib.rcParams['text.usetex'] = True

from astropy import units as u

from matplotlib import gridspec
from matplotlib import pyplot as plt
from matplotlib import ticker, rc
from matplotlib.ticker import NullFormatter
from mpl_toolkits.axes_grid1 import make_axes_locatable

from IPython.display import HTML, display

import matplotlib.image as mgimg

from astropy.io import fits as fits
from astropy.table import Table
from astropy.time import Time, TimeDelta

import imageio as io


import logging

import rfi
rfi = rfi.rfi()
import rfinder_stats as rfi_stats
rfiST = rfi_stats.rfi_stats()


class rfi_plots:

    def __init__(self):

        #set self.logger
        self.logger = logging.getLogger('log-rfinder.log')
        #self.logger.setLevel(logging.INFO)

        #fh = logging.FileHandler('log-rfinder.log')
        #fh.setLevel(logging.INFO)

        #ch = logging.StreamHandler()
        #ch.setLevel(logging.WARNING)

        #formatter = logging.Formatter('%(levelname)s - %(filename)s - %(message)s')
        #fh.setFormatter(formatter)
        #ch.setFormatter(formatter)

        #self.logger.addHandler(ch)
        #self.logger.addHandler(fh)

    # Project spherical coordinates on XY plane
    def aitoff(self,x,y):
        x=x/180*np.pi
        y=y/180*np.pi
        a=np.arccos(np.cos(y)*np.cos(x/2))
        x=2*np.cos(y)*np.sin(x/2)/np.sin(a)*a/np.pi*180
        y=np.sin(y)/np.sin(a)*a/np.pi*180
        
        return x,y

    def plot_rfi_imshow(self,cfg_par,time_step=-1):      
        '''
        
        Plots the .fits image output of rfi_im jpg format.
        
        '''        
        
        #check if image exists
        #plot image

        self.logger.info("\t ... Plotting RFI in 2D ... \n")


        if time_step != -1:
            outputdir = cfg_par['general']['rfitimedir']
            plotdir = cfg_par['general']['timeplotdir2D']
            time_delta = float(cfg_par['rfi']['chunks']['time_step'])*time_step
            time_tmp = int(time_delta)
            if time_tmp == 0:
                time_name = '00'+str(time_tmp)+'m'
            elif time_tmp <100:
                time_name = '0'+str(time_tmp)+'m'
            else:
                time_name= str(time_tmp)+'m'    
        else:
            outputdir = cfg_par['general']['rfidir']
            plotdir = cfg_par['general']['plotdir']        
            time_name = 'full' 

        rfi_freq_base = cfg_par['general']['rfidir']+'freq_base_'+time_name+'_im.fits'   

        if cfg_par['rfi']['RFInder_mode']== 'use_flags':
            rfi_freq_base =outputdir+'flags_base_'+time_name+'.fits'
            rfi_freq_base_plot = plotdir+'flags_base_'+time_name+'.png'
        if cfg_par['rfi']['RFInder_mode']== 'rms_clip':
            rfi_freq_base =outputdir+'rfi_base_'+time_name+'.fits'
            rfi_freq_base_plot = plotdir+'rfi_base_'+time_name+'.png'
        
        #open file
        if os.path.exists(rfi_freq_base) == False:
            self.logger.error('### 2D RFI in fits format does not exist ###')    
        else:
            # open file
            t = fits.open(rfi_freq_base)
            data = t[0].data
            prihdr= t[0].header
            freqs = (np.linspace(1, data.shape[1], data.shape[1])- prihdr['CRPIX1'])*prihdr['CDELT1'] + prihdr['CRVAL1']
            
            # set x-y axes
            bandwidth= (cfg_par['rfi']['highfreq'] - cfg_par['rfi']['lowfreq'] )/1e6
            if bandwidth <=25:
                step_bin = 5
            elif bandwidth <=50 and bandwidth >25:
                step_bin = 10
            elif bandwidth <= 150. and bandwidth > 50.:
                step_bin=25.
            elif bandwidth <= 500. and bandwidth >  150.:
                step_bin=50.                
            else:
                step_bin=100.
            # Smaller multiple 
            freq_max = (freqs[-1] // step_bin) * step_bin  
            # Larger multiple 
            freq_min= (freqs[0] // step_bin) * step_bin + step_bin
            freqs_plot_bin=np.arange(freq_min,freq_max+step_bin,step_bin)
            freqs_plot_bin=np.round(freqs_plot_bin,0)
            freqs_plot_bin=np.array(freqs_plot_bin,dtype=int)

            freqs_plot_idx=np.zeros(freqs_plot_bin.shape)
            for i in xrange(0,len(freqs_plot_bin)):
                idx = (np.abs(freqs_plot_bin[i]-freqs)).argmin()
                freqs_plot_idx[i]=idx

            tele= cfg_par['general']['telescope']
            if tele == 'meerkat' or tele == 'MeerKAT' or tele == 'meerKAT' or tele == 'meer':    
                input_baselines = np.zeros(6)
                for i in xrange (0,len(input_baselines)):
                    input_baselines[i] = 100.*np.power(2,i)
            elif tele == 'apertif' or tele == 'Apertif' or tele == 'APERTIF' or tele == 'wsrt':
                input_baselines = np.zeros(7)
                for i in xrange (0,len(input_baselines)):
                    input_baselines[i] = 50.*np.power(2,i)


            input_baselines_idx=np.zeros(input_baselines.shape)

            baselines = np.array([cfg_par['rfi']['baseline_lenghts']])+0.
            
            if input_baselines[-1] > baselines[0,-1]:
                input_baselines[-1]=np.round(baselines[0,-1],0)

            for i in xrange(0, len(input_baselines)):
                idx = (np.abs(input_baselines[i] - baselines[0,:])).argmin()
                input_baselines_idx[i]=idx
            
            #set rc parameters 
            plt.rcParams['image.interpolation']='nearest'
            plt.rcParams['image.origin']='lower'
            plt.rcParams['image.aspect']='auto'

            params = {'font.family'         :' serif',
                      'font.style'          : 'normal',
                      'font.weight'         : 'book',
                      'font.size'           : 18.0,
                      'axes.linewidth'      : 1,
                      'lines.linewidth'     : 1,
                      'xtick.labelsize'     : 16,
                      'ytick.labelsize'     : 16, 
                      'xtick.direction'     :'in',
                      'ytick.direction'     :'in',
                      #'xtick.top'           : True,   # draw ticks on the top side
                      #'xtick.bottom'        : True,   # draw ticks on the bottom side    
                      #'ytick.left'          : True,   # draw ticks on the top side
                      #'ytick.right'         : True,   # draw ticks on the bottom side  
                      'xtick.major.size'    : 4,
                      'xtick.major.width'   : 1,
                      'xtick.minor.size'    : 2,
                      'xtick.minor.width'   : 1,
                      'ytick.major.size'    : 4,
                      'ytick.major.width'   : 1,
                      'ytick.minor.size'    : 2,
                      'ytick.minor.width'   : 1, 
                      'text.usetex'         : True,
                      'text.latex.unicode'  : True
                       }
            plt.rcParams.update(params)
            # Format axes
            nullfmt        = NullFormatter() 
            left, width    = 0.05, 0.9                                                                     #|These determine where the subplots go
            bottom, height = 0.1, 0.85
            #left_h = left+width+0.015
            #bottom_h = left+height+0.015

            box_centre    = [left, bottom, width, height]
            #box_x      = [left, bottom_h, width, 0.12]
            #box_y      = [left_h, bottom, 0.12, height]
            #box_cbar       = [left_h+0.13, bottom, 0.05, height]

            fig = plt.figure(1, figsize=(12,8))

            ax = plt.axes(box_centre)
            
            if bandwidth <= 500.:
                colormap = 'nipy_spectral_r'
            else:
                colormap = 'afmhot'


            im = ax.imshow(data,cmap=colormap,vmin=0,vmax=100)

            # ticks & labels
            ax.set_xticks(freqs_plot_idx)
            ax.set_yticks(input_baselines_idx)

            ax.set_xticklabels(freqs_plot_bin)
            ax.set_yticklabels(input_baselines)
                    
            ax.set_xlabel('Frequency [MHz]')
            ax.set_ylabel('Baseline lenght [m]')
            
            # colorbar

            divider = make_axes_locatable(ax)
            cax = divider.append_axes("right", size="3%", pad=0.05)
            cbar = fig.colorbar(im,ticks=[0,10,20, 30,40,50,60,70,80,90,100], cax=cax ) 

            
            if cfg_par['rfi']['RFInder_mode']== 'rms_clip':
                cbar.set_label(r'$\% > 5 \times$ r.m.s.')
            if cfg_par['rfi']['RFInder_mode']== 'use_flags':
                cbar.set_label(r'$\%$ flagged visibilites')

            #times, start, end = rfi.time_chunk(cfg_par)
            #start.format='iso' 
            #start.subformat='date_hm'   
            #end.format='iso'
            #end.subformat='date_hm'


            if cfg_par['rfi']['chunks']['time_enable']== False:
                start = cfg_par['rfi']['startdate']
                end = cfg_par['rfi']['enddate']

            elif cfg_par['rfi']['chunks']['time_enable'] == True:
                time_del = TimeDelta(time_delta*60., format='sec')
                time_delta_plus = TimeDelta(float(cfg_par['rfi']['chunks']['time_step'])*60., format='sec')
                start = cfg_par['rfi']['startdate']+time_del
                end = start+time_delta_plus
           
            pol = str(cfg_par['rfi']['polarization'] )

            if cfg_par['rfi']['RFInder_mode']== 'rms_clip':
                rfi_clip = str(cfg_par['rfi']['rms_clip'])+r'$\sigma$ clip'
                title_plot = 'Stokes:  {0:s} / {1:s} / {2:%d}{2:%b}{2:%y}: {2:%H}:{2:%M} - {3:%H}:{3:%M}'.format(pol,rfi_clip,start.datetime,end.datetime)
            if cfg_par['rfi']['RFInder_mode']== 'use_flags':
                title_plot = 'Stokes:  {0:s} / {1:s} / {2:%d}{2:%b}{2:%y}: {2:%H}:{2:%M} - {3:%H}:{3:%M}'.format(pol,'Flags',start.datetime,end.datetime)
            
            ax.set_title(title_plot)
         
            plt.savefig(rfi_freq_base_plot,format='png' ,overwrite=True)
            plt.close(fig)        
            self.logger.info("\t ... RFI per baseline lenght and frequency plotted ... \n\n")

    def plot_noise_frequency(self,cfg_par,time_step=-1):
        '''
        Plots the noise or % of rfi per frequency channel for all, long and short baselines.
        In default.cfga
            aperfi_noise = 'rfi' (or flag)
            aperfi_plot_long_short = False
        '''

        #open file
        #if os.path.exists(self.rfi_table) == False:
        #    self.self.logger.error('### Table of RFI and flags of visibilities does not exist ###')    
        #    self.self.logger.error('### Run aperfi.rfi_frequency() first ###')  
        #else:  

        self.logger.info("\t ... Plotting RFI in 1D ... \n")


        table_tmp = string.split(cfg_par['general']['msname'][0],'.MS')
        if len(table_tmp) == 1:
            table_tmp = string.split(cfg_par['general']['msname'][0],'.ms')

        if time_step != -1:
            tabledir = cfg_par['general']['timetabledir'] 
            plotdir = cfg_par['general']['timeplotdir1D']
            time_delta = float(cfg_par['rfi']['chunks']['time_step'])*time_step
            time_tmp = int(time_delta)
            #name file according to WSRT beams
            if time_tmp == 0:
                time_name = '00'+str(time_tmp)+'m'
            elif time_tmp <100:
                time_name = '0'+str(time_tmp)+'m'
            else:
                time_name= str(time_tmp)+'m'    
        else:
            tabledir = cfg_par['general']['tabledir']        
            plotdir = cfg_par['general']['plotdir']        
            time_name = 'full' 

        if cfg_par['rfi']['RFInder_mode']== 'use_flags':
            table_name = str(table_tmp[0])+'_flags_'+time_name+'.fits'
        if cfg_par['rfi']['RFInder_mode']== 'rms_clip':
            table_name = str(table_tmp[0])+'_rfi_'+time_name+'.fits'

        rfi_table = tabledir+table_name
        
        #open file
        if os.path.exists(rfi_table) == False:
            self.logger.error('### Table of RFI results does not exist ###')    
        else:    
            
            t = fits.open(rfi_table)
            data_vec = t[1].data
            cols = t[1].columns
            
            freqs = np.array(data_vec['frequency'],dtype=float)
            flags = np.array(data_vec['percentage_flags'],dtype=float)
            noise_factor = np.array(data_vec['noise_factor'],dtype=float)
            noise_factor_long = np.array(data_vec['noise_factor_long'],dtype=float)
            flags_long = np.array(data_vec['percentage_flags_long'],dtype=float)
            noise_factor_short = np.array(data_vec['noise_factor_short'],dtype=float)
            flags_short = np.array(data_vec['percentage_flags_short'],dtype=float)

           
            if cfg_par['plots']['plot_noise'] == 'noise':
                rms = np.array(cfg_par['rfi']['theo_rms']*1e3,dtype=float)
                noise_all = noise_factor*rms
                noise_short = noise_factor_short*rms
                noise_long = noise_factor_long*rms
                out_plot = plotdir+'noise_'+time_name
            
            if cfg_par['plots']['plot_noise'] == 'noise_factor':
                self.logger.info("\t ... Plotting factor of noise increas per frequency channel ...")
                noise_all = noise_factor
                noise_short = noise_factor_short
                noise_long = noise_factor_long    
                out_plot = plotdir+'noisefactor_'+time_name
            
            if cfg_par['plots']['plot_noise'] == 'rfi':
                self.logger.info("\t ... Plotting percentage of flagged RFI per frequency channel ...")
                noise_all = flags
                noise_long = flags_long
                noise_short = flags_short
                out_plot = plotdir+'flags_'+time_name


                    # initialize plotting parameters
            params = {'font.family'         :' serif',
                      'font.style'          : 'normal',
                      'font.weight'         : 'book',
                      'font.size'           : 18.0,
                      'axes.linewidth'      : 1,
                      'lines.linewidth'     : 1,
                      'xtick.labelsize'     : 16,
                      'ytick.labelsize'     : 16, 
                      'xtick.direction'     :'in',
                      'ytick.direction'     :'in',
                      #'xtick.top'           : True,   # draw ticks on the top side
                      #'xtick.bottom'        : True,   # draw ticks on the bottom side    
                      #'ytick.left'          : True,   # draw ticks on the top side
                      #'ytick.right'         : True,   # draw ticks on the bottom side  
                      'xtick.major.size'    : 4,
                      'xtick.major.width'   : 1,
                      'xtick.minor.size'    : 2,
                      'xtick.minor.width'   : 1,
                      'ytick.major.size'    : 4,
                      'ytick.major.width'   : 1,
                      'ytick.minor.size'    : 2,
                      'ytick.minor.width'   : 1, 
                      'text.usetex'         : True,
                      'text.latex.unicode'  : True
                       }
            plt.rcParams.update(params)
            #plt.rc('xtick', labelsize=20)

            # Format axes
            nullfmt        = NullFormatter() 
            left, width    = 0.05, 0.9                                                                     #|These determine where the subplots go
            bottom, height = 0.1, 0.85
            #left_h = left+width+0.015
            #bottom_h = left+height+0.015

            box_centre    = [left, bottom, width, height]
                    
            # initialize figure
            fig = plt.figure(1, figsize=(12,8))
            plt.rc('xtick', labelsize=20)

            # Initialize subplots
            ax1 = plt.axes(box_centre)
            ax1.set_xlabel(r'Frequency [MHz]')
          
            bandwidth= (cfg_par['rfi']['highfreq'] - cfg_par['rfi']['lowfreq'] )/1e6

            if bandwidth <= 150.:
                step_bin=25.
            elif bandwidth <= 500. and bandwidth >  150.:
                step_bin=50.                
            else:
                step_bin=100.

            # Smaller multiple 
            freq_max = (freqs[-1] // step_bin) * step_bin  
            # Larger multiple 
            freq_min= (freqs[0] // step_bin) * step_bin + step_bin

            freqs_plot_bin=np.arange(freq_min,freq_max+step_bin,step_bin)
            freqs_plot_bin=np.round(freqs_plot_bin,0)
            freqs_plot_bin=np.array(freqs_plot_bin,dtype=int)

            #for i in xrange(0, len(freqs_plot_bin)):
            #    idx = (np.abs(freqs_plot_bin[i] - freqs)).argmin()
            #    freqs_plot_idx[i]=idx
            ax1.set_xticks(freqs_plot_bin)
            ax1.set_xticklabels(freqs_plot_bin)
            ax1.set_xlim([(cfg_par['rfi']['lowfreq']-5.*cfg_par['rfi']['chan_widths'])/1e6,
                (cfg_par['rfi']['highfreq']+5.*cfg_par['rfi']['chan_widths'])/1e6])
           
            if cfg_par['plots']['plot_noise'] != 'rfi':
                ax1.set_yscale('log', basey=10)
                if np.isnan(np.sum(noise_all)):
                    noise_all=np.zeros([len(freqs)])+100.
                if np.isnan(np.sum(noise_short)):
                    noise_short=np.zeros([len(freqs)])+100.
                if np.isnan(np.sum(noise_long)):
                    noise_long= np.zeros([len(freqs)])+100.

            #define title output                     
            
            #plot
            label_all = 'All baselines' 
            label_long = r'Baselines $>$ '+str(cfg_par['rfi']['baseline_cut'])+' m'
            label_short = r'Baselines $<$ '+str(cfg_par['rfi']['baseline_cut'])+' m' 

            if cfg_par['plots']['plot_long_short'] == True:
                self.logger.info("\t ... Plotting RFI in long and short baselines ...")
                ax1.step(freqs,noise_short, where= 'pre', color='red', linestyle='-',label=label_short)
                ax1.step(freqs,noise_long, where= 'pre', color='blue', linestyle='-',label=label_long)
                out_plot = out_plot+'_sl'

            ax1.step(freqs,noise_all, where= 'pre', color='black', linestyle='-',label=label_all)

            #titleplot = self.target+': '+self.aperfi_startime+' - '+self.aperfi_endtime
            #ax1.set_title(titleplot)
            
            # set axis, legend ticks

            #ax1.set_xlim([np.min(freqs)-5,np.max(freqs)+5])

            #xticks_num = np.linspace(cfg_par['rfi']['lowfreq'],cfg_par['rfi']['highfreq'],10,dtype=int)
            #ax1.set_xticks(xticks_num)

            if cfg_par['plots']['plot_noise']  == 'noise_factor':
                ax1.set_yticks([1,round(np.sqrt(2),2),2,3,5,10,50]) 
                ax1.set_yscale('linear')     
                ax1.set_ylabel(r'Factor of noise increase')
                ax1.get_yaxis().set_major_formatter(ticker.ScalarFormatter())

            if cfg_par['plots']['plot_noise'] == 'noise':
                ax1.set_yscale('linear')     
                ax1.set_ylabel(r'Predicted noise [mJy beam$^{-1}$]')     
                ax1.get_yaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())

            if cfg_par['plots']['plot_noise'] == 'rfi':
                if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                    ax1.set_ylabel(r'$\% > 5 \times$ r.m.s.')
                if cfg_par['rfi']['RFInder_mode'] == 'use_flags':
                    ax1.set_ylabel(r'$\%$ flagged visibilites')    
                ax1.set_ylim([-5,105]) 
                ax1.set_yticks([0,20,40,60,80,100]) 
            
            legend = plt.legend(loc=1)
            legend.get_frame().set_edgecolor('black')

            if cfg_par['rfi']['chunks']['time_enable']== False:
                start = cfg_par['rfi']['startdate']
                end = cfg_par['rfi']['enddate']

            elif cfg_par['rfi']['chunks']['time_enable'] == True:
                time_del = TimeDelta(time_delta*60., format='sec')
                time_delta_plus = TimeDelta(float(cfg_par['rfi']['chunks']['time_step'])*60., format='sec')
                start = cfg_par['rfi']['startdate']+time_del
                end = start+time_delta_plus
            
            pol =str(cfg_par['rfi']['polarization']) 
            if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                rfi_clip = str(cfg_par['rfi']['rms_clip'])+r'$\sigma$ clip'        
                title_plot = 'Stokes: {0:s} / {1:s} / {2:%d}{2:%b}{2:%y}: {2:%H}:{2:%M} - {3:%H}:{3:%M}'.format(pol,rfi_clip,start.datetime,end.datetime)
            
            if cfg_par['rfi']['RFInder_mode'] == 'use_flags':
                title_plot = 'Stokes: {0:s} / {1:s} / {2:%d}{2:%b}{2:%y}: {2:%H}:{2:%M} - {3:%H}:{3:%M}'.format(pol,'Flags',start.datetime,end.datetime)
            
            ax1.set_title(title_plot)
            ax1.minorticks_on()
            ax1.minorticks_on()
            # Save figure to file
            if cfg_par['rfi']['RFInder_mode']== 'use_flags':
                rfi_freq_plot = out_plot+'_flags.png'
            if cfg_par['rfi']['RFInder_mode']== 'rms_clip':
                rfi_freq_plot = out_plot+'_rfi.png'

            plt.savefig(rfi_freq_plot,format='png',overwrite = True)      
            plt.close(fig)        
            self.logger.info("\t ... RFI in 1D plotted ...\n\n")
       

    def plot_altaz(self,cfg_par,number_chunks):
        '''
        Plots the elevation/azimuth of the observation scan binned by time chunks, for every binned spectral window
        '''


        self.logger.info("\t ... Plotting Alt/Az for binned dataset ... \n")
      
        if os.path.exists(cfg_par['general']['timetabledir']) == False:
            self.logger.error("\t Folder with time subsets missing")

        table_tmp = string.split(cfg_par['general']['msname'][0],'.MS')
        if len(table_tmp) == 1:
            table_tmp = string.split(cfg_par['general']['msname'][0],'.ms')

        freq_S = cfg_par['rfi']['lowfreq']/1e6
        freq_E = cfg_par['rfi']['highfreq']/1e6

        step_bin = cfg_par['rfi']['chunks']['spw_width']
        
        freqs_bin=np.arange(freq_S,freq_E+step_bin,step_bin)

        pol = cfg_par['rfi']['polarization'] 

        for j in xrange(0,len(freqs_bin)):
            spw = np.full((number_chunks),np.nan)
            azimuth = np.full((number_chunks),np.nan)
            altitude= np.full((number_chunks),np.nan)
            flags = np.full((number_chunks),np.nan)

            for i in xrange(0,number_chunks):
            
                tabledir = cfg_par['general']['timetabledir'] 
                time_delta = float(cfg_par['rfi']['chunks']['time_step'])*i
                time_tmp = int(time_delta)

                #name file according to WSRT beams
                if time_tmp == 0:
                    time_name = '00'+str(time_tmp)+'m'
                elif time_tmp <100:
                    time_name = '0'+str(time_tmp)+'m'
                else:
                    time_name= str(time_tmp)+'m'    

                if cfg_par['rfi']['RFInder_mode']== 'use_flags':
                    table_name = str(table_tmp[0])+'_flags_'+time_name+'_spwbin.fits'
                elif cfg_par['rfi']['RFInder_mode']== 'rms_clip':
                    table_name = str(table_tmp[0])+'_rfi_'+time_name+'_spwbin.fits'

                rfi_table = tabledir+table_name

                #open file
                if os.path.exists(rfi_table) == False:
                    self.logger.error('### Table of RFI results does not exist ###')    
                    continue         
                else:    
                    table = Table.read(rfi_table)
                    spw[i] = table['frequency'][j]
                    azimuth[i] = table['azimuth'][j]
                    altitude[i] = table['altitude'][j]
                    flags[i] = table['percentage_flags'][j]
            #az=np.array([az],dtype=float)
            #alt=np.array([alt],dtype=float)
            #az,alt=self.aitoff(az,alt)
            #azimuth=np.array(azimuth,dtype=float)
            #altitude=np.array(altitude,dtype=float)

            azimuth,altitude =self.aitoff(azimuth-180.,altitude)
            #azimuth=np.array([azimuth],dtype=float)
            #altitude=np.array([altitude],dtype=float)

            plotdir = cfg_par['general']['altazplotdir']

            start_freq = int(np.round(freq_S+float(cfg_par['rfi']['chunks']['spw_width'])*j,0))
            end_freq = int(np.round(start_freq+float(cfg_par['rfi']['chunks']['spw_width']),0))

            spwname= str(start_freq)+'-'+str(end_freq)

            # Save figure to file
            if cfg_par['rfi']['RFInder_mode']== 'use_flags':
               altazplot = plotdir+'AltAZ_flags'+spwname+'MHz.png'
            if cfg_par['rfi']['RFInder_mode']== 'rms_clip':
               altazplot = plotdir+'AltAZ_rfi'+spwname+'MHz.png'

                    # initialize plotting parameters
            params = {'font.family'         :' serif',
                      'font.style'          : 'normal',
                      'font.weight'         : 'book',
                      'font.size'           : 18.0,
                      'axes.linewidth'      : 1,
                      'lines.linewidth'     : 1,
                      'xtick.labelsize'     : 16,
                      'ytick.labelsize'     : 16, 
                      'xtick.direction'     :'in',
                      'ytick.direction'     :'in',
                      #'xtick.top'           : True,   # draw ticks on the top side
                      #'xtick.bottom'        : True,   # draw ticks on the bottom side    
                      #'ytick.left'          : True,   # draw ticks on the top side
                      #'ytick.right'         : True,   # draw ticks on the bottom side  
                      'xtick.major.size'    : 4,
                      'xtick.major.width'   : 1,
                      'xtick.minor.size'    : 2,
                      'xtick.minor.width'   : 1,
                      'ytick.major.size'    : 4,
                      'ytick.major.width'   : 1,
                      'ytick.minor.size'    : 2,
                      'ytick.minor.width'   : 1, 
                      'text.usetex'         : True,
                      'text.latex.unicode'  : True
                       }
            plt.rcParams.update(params)
            #plt.rc('xtick', labelsize=20)
            bandwidth= (cfg_par['rfi']['highfreq'] - cfg_par['rfi']['lowfreq'] )/1e6

            if bandwidth <= 500.:
                colormap = 'nipy_spectral_r'
            else:
                colormap = 'afmhot'

            # Format axes
            nullfmt        = NullFormatter() 
            left, width    = 0.12, 0.64                                                                     #|These determine where the subplots go
            bottom, height = 0.12, 0.54
            left_h = left+width+0.015
            bottom_h = left+height+0.015

            box_centre    = [left, bottom, width, height]
            box_x      = [left, bottom_h, width, 0.12]
            box_y      = [left_h, bottom, 0.12, height]
            box_cbar       = [left_h+0.13, bottom, 0.05, height]

            fig = plt.figure(1, figsize=(12,8))

            ax_centre = plt.axes(box_centre)
            ax_x = plt.axes(box_x)
            ax_y = plt.axes(box_y)
            ax_cbar   = plt.axes(box_cbar)

            # Remove labels from supplementary plots
            ax_x.xaxis.set_major_formatter(nullfmt)
            ax_y.yaxis.set_major_formatter(nullfmt)
            ax_cbar.xaxis.set_major_formatter(nullfmt)
            ax_cbar.yaxis.set_major_formatter(nullfmt)
            ax_cbar.set_visible(False)

            # Add minor tick marks
            ax_centre.minorticks_on()
            ax_x.minorticks_on()
            ax_y.minorticks_on()

            # initialize figure
            #fig = plt.figure(figsize =(8,8))
            #fig.subplots_adjust(hspace=0.0)
            #gs = gridspec.GridSpec(1, 1)

            # Initialize subplots
            #ax1 = fig.add_subplot(gs[0])
            #centre
            ax_centre.set_xlabel(r'Azimuth [deg]')
            ax_centre.set_ylabel(r'Altitude [deg]',labelpad=15)
            
            ax_centre.set_ylim([0,90])
            ax_centre.set_xlim([-180.,180])
            ax_centre.set_xticks([-180,-135,-90,-45,0,45,90,135,180])        
            #ax_centre.set_xticklabels(['0','45','90','135','180','225','270','315','0'])
            #ax_centre.set_yticks([0,10,20,30,40,50,60,70,80,90])
            #ax_centre.set_xticks([])        
            ax_centre.set_yticks([])

            asse = ax_centre.scatter(azimuth,altitude,c=flags,cmap=colormap,vmin=0,vmax=100.,s=100,linewidth=0.5,edgecolors='black',label=r'Stokes: {0:s}'.format(pol))

            azs=np.arange(-180.,181.,45.)
            als=np.arange(0.,91.,1.)
            azs[azs==0]=1e-6
            als[als==0]=1e-6
            for counter in azs:
                [X,Y]=self.aitoff(counter*np.ones(als.shape),als)
                ax_centre.plot(X,Y,'k-',linewidth=0.5)

            azs=np.arange(-180.,181.,1.)
            als=np.arange(0.,91.,15.)
            azs[azs==0]=1e-6
            als[als==0]=1e-6
            for al in als:
                [X,Y]=self.aitoff(azs,al*np.ones(azs.shape))
                ax_centre.plot(X,Y,'k-',linewidth=0.5)

            for dd in [15.,30.,45.,60.,75.]:
                [X,Y]=self.aitoff(-180.,dd)
                ax_centre.text(X,Y,r'${0:s}$'.format(str(int(round(dd,0)))),horizontalalignment='right',fontsize='medium')

            legend = ax_centre.legend(loc=1,fontsize=14,handletextpad=0.1,borderpad=0.2)
            legend.get_frame().set_edgecolor('black')
            legend.legendHandles[0].set_color('black')
            legend.legendHandles[0]._sizes = [35]

            start = cfg_par['rfi']['startdate']
            end = cfg_par['rfi']['enddate']
            
             # colorbar
            cbar = plt.colorbar(asse,ax=ax_cbar, fraction=1.0,ticks=[0,20,40,60,80,100]) 
            if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                cbar.set_label(r'$\% > 5 \times$ r.m.s.')
            if cfg_par['rfi']['RFInder_mode'] == 'use_flags':
                cbar.set_label(r'$\%$ flagged visibilites')       
            #title
            if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                rfi_clip = str(cfg_par['rfi']['rms_clip'])+r'$\sigma$ clip'        
                title_plot = '{0:d}-{1:d} MHz / {2:s} / {3:%d}{3:%b}{3:%y}: {3:%H}:{3:%M} - {4:%H}:{4:%M}'.format(start_freq,end_freq,rfi_clip,start.datetime,end.datetime)
            if cfg_par['rfi']['RFInder_mode'] == 'use_flags':
                title_plot = '{0:d}-{1:d} MHz / {2:s} / {3:%d}{3:%b}{3:%y}: {3:%H}:{3:%M} - {4:%H}:{4:%M}'.format(start_freq,end_freq,'Flags',start.datetime,end.datetime)        

            #x plot
            #ax_x.set_xlabel(r'Azimuth [deg]',fontsize=16)
            if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                ax_x.set_ylabel(r'$\%$ RFI')
            if cfg_par['rfi']['RFInder_mode'] == 'use_flags':
                ax_x.set_ylabel(r'$\%$ flagged')                
            ax_x.set_ylim([-5,100])
            ax_x.set_xlim([-180,180])
            ax_x.set_yticks([0,25,50,75,100])
   
            ax_x.set_title(title_plot)
           
            ax_x.scatter(azimuth,flags,c=flags,cmap=colormap,vmin=0,vmax=100.,s=70,linewidth=0.5,edgecolors='black')
            
            #y plot
            #ax_x.set_xlabel(r'Azimuth [deg]',fontsize=16)
            if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                ax_y.set_xlabel(r'$\%$ RFI ')
            if cfg_par['rfi']['RFInder_mode'] == 'use_flags':
                ax_y.set_xlabel(r'$\%$ flagged')          

            ax_y.set_xlim([-5,100])
            ax_y.set_ylim([0,90])
            ax_y.set_xticks([0,25,50,75,100])
            ax_y.scatter(flags,altitude,c=flags,cmap=colormap,vmin=0,vmax=100.,s=70,linewidth=0.5,edgecolors='black')


            # Finish everything up
            plt.savefig(altazplot,format='png',overwrite = True)
            plt.close()
            self.logger.info(('\t ... ALT/AZ for spw: {0:d}-{1:d} MHz  ...\n').format(start_freq,end_freq))

        return 0

    def plot_altaz_short(self,cfg_par):
        '''
        Plots the elevation/azimuth of the observation scan binned by time chunks, for every binned spectral window
        '''


        self.logger.info("\t ... Plotting Alt/Az for observation ... \n")
        
        times = cfg_par['rfi']['times']
        azimuth = []
        altitude = []

        for i in xrange(0,len(times)):
        
            altaz = rfiST.alt_az(cfg_par,times[i])
            azimuth.append(altaz.az)
            altitude.append(altaz.alt)


        azimuth=np.array([azimuth],dtype=float)
        altitude=np.array([altitude],dtype=float)

        [azimuth,altitude] =self.aitoff(azimuth-180.,altitude)


        plotdir = cfg_par['general']['plotdir']

        freq_S = cfg_par['rfi']['lowfreq']/1e6
        freq_E = cfg_par['rfi']['highfreq']/1e6

        start_freq = int(np.round(freq_S,0))
        end_freq = int(np.round(freq_E,0))

        spwname= str(start_freq)+'-'+str(end_freq)

        # Save figure to file
        altazplot = cfg_par['general']['plotdir']+'AltAZ_full.png'

                # initialize plotting parameters
        params = {'font.family'         :' serif',
                  'font.style'          : 'normal',
                  'font.weight'         : 'book',
                  'font.size'           : 18.0,
                  'axes.linewidth'      : 1,
                  'lines.linewidth'     : 1,
                  'xtick.labelsize'     : 16,
                  'ytick.labelsize'     : 16, 
                  'xtick.direction'     :'in',
                  'ytick.direction'     :'in',
                  'xtick.major.size'    : 4,
                  'xtick.major.width'   : 1,
                  'xtick.minor.size'    : 2,
                  'xtick.minor.width'   : 1,
                  'ytick.major.size'    : 4,
                  'ytick.major.width'   : 1,
                  'ytick.minor.size'    : 2,
                  'ytick.minor.width'   : 1, 
                  'text.usetex'         : True,
                  'text.latex.unicode'  : True
                   }
        plt.rcParams.update(params)

        # Format axes
        nullfmt        = NullFormatter() 
        left, width    = 0.05, 0.9                                                                     #|These determine where the subplots go
        bottom, height = 0.1, 0.85

        box_centre    = [left, bottom, width, height]
        fig = plt.figure(1, figsize=(12,8))

        ax_centre = plt.axes(box_centre)
        # Add minor tick marks
        ax_centre.minorticks_on()

        ax_centre.set_xlabel(r'Azimuth [deg]')
        ax_centre.set_ylabel(r'Altitude [deg]')
        
        ax_centre.set_ylim([0,90])
        ax_centre.set_xlim([-180.,180])
        ax_centre.set_xticks([-180,-135,-90,-45,0,45,90,135,180])        
        ax_centre.set_yticks([])

        asse = ax_centre.scatter(azimuth,altitude,c='black',s=100,linewidth=0.5,edgecolors='black',label=r'${0:s}$'.format(cfg_par['general']['fieldname']))

        azs=np.arange(-180.,181.,45.)
        als=np.arange(0.,91.,1.)
        azs[azs==0]=1e-6
        als[als==0]=1e-6
        for counter in azs:
            [X,Y]=self.aitoff(counter*np.ones(als.shape),als)
            ax_centre.plot(X,Y,'k-',linewidth=0.5)

        azs=np.arange(-180.,181.,1.)
        als=np.arange(0.,91.,15.)
        azs[azs==0]=1e-6
        als[als==0]=1e-6
        for al in als:
            [X,Y]=self.aitoff(azs,al*np.ones(azs.shape))
            ax_centre.plot(X,Y,'k-',linewidth=0.5)

        for dd in [15.,30.,45.,60.,75.]:
            [X,Y]=self.aitoff(-180.,dd)
            ax_centre.text(X,Y,r'${0:s}$'.format(str(int(round(dd,0)))),horizontalalignment='right',fontsize='medium')

        legend = ax_centre.legend(loc=1,fontsize=14,handletextpad=0.1,borderpad=0.2)
        legend.get_frame().set_edgecolor('black')
        legend.legendHandles[0].set_color('black')
        legend.legendHandles[0]._sizes = [35]

        start = cfg_par['rfi']['startdate']
        end = cfg_par['rfi']['enddate']

        title_plot = '{0:d}-{1:d} MHz / {2:%d}{2:%b}{2:%y}: {2:%H}:{2:%M} - {3:%H}:{3:%M}'.format(start_freq,end_freq,start.datetime,end.datetime)
         
        ax_centre.set_title(title_plot)

        # Finish everything up
        plt.savefig(altazplot,format='png',overwrite = True)
        plt.close()
        self.logger.info(('\t ... ALT/AZ for {0:s} ...\n').format(cfg_par['general']['fieldname']))

        return 0
    

    def gif_me_up(self,cfg_par,filenames,outmovie):
        

        self.logger.info(('\t ... Creating movie ...'))
       
        # initiate an empty  list of "plotted" images 
        images=[]

        for filename in filenames:
            images.append(io.imread(filename))
            
        io.mimsave(outmovie,images,duration=1.5)


        self.logger.info(('\t ... Movie written to file ...\n'))

        return 0





# Import modules
import sys, string, os
import numpy as np
import yaml
import json
import glob
#import pyrap.tables as tables
import casacore.tables as tables
import logging


from astropy.time import Time
from astropy.coordinates import SkyCoord
import numpy as np
from astropy import units as u

from astropy.io import fits, ascii
from astropy import units as u
from astropy.table import Table, Column, MaskedColumn
import rfinder

import rfinder_stats as rfi_stats
import rfinder_files as rfi_files


rfiST = rfi_stats.rfi_stats()
#rfiPLOT = rfi_plots.rfi_plots()


class rfi:

    def __init__(self):

        self.logger = logging.getLogger('log-rfinder.log')
        #self.logger.setLevel(logging.INFO)

        #fh = logging.FileHandler('log-rfinder.log')
        #fh.setLevel(logging.INFO)

        #ch = logging.StreamHandler()
        #ch.setLevel(logging.WARNING)

        #formatter = logging.Formatter('%(levelname)s - %(filename)s - %(message)s')
        #fh.setFormatter(formatter)
        #ch.setFormatter(formatter)

        #self.logger.addHandler(ch)
        #self.logger.addHandler(fh)


    def load_from_ms(self,cfg_par,times=0,counter=0):
        '''

        Loads important columns from MS file
        From MS: 
        Field_ID, Data, Flag, Antenna1, Antenna2,
        From MS/ANTENNA:
        Position,Name
        From MS/SPECTRAL_WINDOW
        Chan_width, Chan_freq

        '''

        if counter == 0 :
            self.logger.warning("\t ... Field, Antenna & Bandwidth Info ...\n")

        self.msfile = cfg_par['general']['msfullpath']
        self.aperfi_badant = cfg_par['rfi']['bad_antenna'] 
        self.selectFieldID = cfg_par['general']['field']
        
        fields=tables.table(self.msfile+'/FIELD')
        self.fieldNames = fields.getcol('NAME')
        selectFieldName= self.fieldNames[int(self.selectFieldID)]
        cfg_par['general']['fieldname'] = selectFieldName
        self.coords=fields.getcol('REFERENCE_DIR')
        self.coords =self.coords*180./np.pi
        cfg_par['rfi']['coords'] = SkyCoord(self.coords[self.selectFieldID,:,0]*u.degree, self.coords[self.selectFieldID,:,1]*u.degree,  unit=(u.deg, u.deg))

        if counter == 0 :
            self.logger.warning("\tField with name {0:s} (Field ID = {1:d})".format(selectFieldName,self.selectFieldID))
        #self.logger.info("\tCoordinates {}".format(selectFieldName,self.selectFieldID))
  
        antennas = tables.table(self.msfile +'/ANTENNA')
        self.ant_pos = np.array(antennas.getcol('POSITION'))
        self.ant_wsrtnames = np.array(antennas.getcol('NAME'))
        
        self.ant_names = np.arange(0,self.ant_wsrtnames.shape[0],1)
        self.nant = len(self.ant_names)

        #logging
        cfg_par['rfi']['nant'] = self.nant
        cfg_par['rfi']['ant_names'] = self.ant_names

        if counter == 0 :
            self.logger.warning("\tTotal number of antennas:\t"+str(self.nant))
            self.logger.warning("\tAntenna names:\t\t"+str(self.ant_names))

        antennas.close()

        spw=tables.table(self.msfile+'/SPECTRAL_WINDOW')
        self.channelWidths=spw.getcol('CHAN_WIDTH')

        self.channelFreqs=spw.getcol('CHAN_FREQ')
        cfg_par['rfi']['chan_widths'] = self.channelWidths[0][0]
        cfg_par['rfi']['lowfreq'] = float(self.channelFreqs[0][0])
        cfg_par['rfi']['highfreq'] = float(self.channelFreqs[-1][-1])

        spw.close()

        if counter == 0 :
       
            self.logger.warning("\tChannel Width [kHz]:\t"+str(cfg_par['rfi']['chan_widths']/1e3))
            self.logger.warning("\tStart         [GHz]:\t"+str(cfg_par['rfi']['lowfreq']/1e9))
            self.logger.warning("\tEnd           [GHz]:\t"+str(cfg_par['rfi']['highfreq']/1e9)+'\n')


        #determine start and end date
        times_tm, start_tmp, end_tmp = rfiST.time_chunk(cfg_par)

        t=tables.table(self.msfile)

        if counter !=0:
            value_end = times[1]
            value_start = times[0]
    
            altaz = rfiST.alt_az(cfg_par,times[0])
            cfg_par['rfi']['altaz'] = altaz
            
            t2 = tables.taql('select from $t where TIME < $value_end and TIME>$value_start')
            self.fieldIDs=t2.getcol('FIELD_ID')
            self.ant1 = t2.getcol('ANTENNA1')
            self.ant2 = t2.getcol('ANTENNA2')

            selection  = self.fieldIDs==self.selectFieldID
            selection *= self.ant1!=self.ant2
            if np.sum(selection) !=0. :
                if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                    self.vis  = t2.getcol('DATA')[selection]
                self.flag = t2.getcol('FLAG')[selection]
                self.interval = t2.getcol('INTERVAL')[selection]
                empty_table=0     
            else:
                self.logger.warning('\t ### Table of selected interval is empty ')
                self.logger.warning('\t     Correct noise_measure_edges in rfi of parameter file ###')
                empty_table=1
            t2.close()
        
        else:

            self.fieldIDs=t.getcol('FIELD_ID')
            self.ant1=t.getcol('ANTENNA1')
            self.ant2=t.getcol('ANTENNA2')

            #select from cross correlations of the correct field
            selection=self.fieldIDs==self.selectFieldID
            selection*=self.ant1!=self.ant2
    
            altaz = rfiST.alt_az(cfg_par,times_tm[0])
            cfg_par['rfi']['altaz'] = altaz

            if np.sum(selection) !=0. :
                if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                    self.vis  = t.getcol('DATA')[selection]
                self.flag = t.getcol('FLAG')[selection]
                self.interval = t.getcol('INTERVAL')[selection]
                empty_table=0     
            else:
                self.logger.warning('\t ### Table of selected interval is empty ')
                if cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                    self.logger.warning('\t     Correct noise_measure_edges in rfi of parameter file ###')
                empty_table=1
        

        t.close()
       
        if not self.aperfi_badant:
            nrbadant =len(int(self.aperfi_badant))
        else:
            nrbadant = 0.

        #visibilities over all times per baseline
        #estimate noise
        if empty_table!=1:
            #determine number of baselines
            nrAnt=np.unique(np.concatenate((self.ant1,self.ant2))).shape[0]
            nrBaseline=(nrAnt-nrbadant)*(nrAnt-nrbadant-1)/2        
            cfg_par['rfi']['number_baseline'] = nrBaseline
            rfiST.predict_noise(cfg_par,self.channelWidths,self.interval,self.flag)
            cfg_par['rfi']['vis_alltimes_baseline'] = self.flag.shape[0]/nrBaseline


        self.logger.info("\t ... info from MS file loaded  \n\n")

        return empty_table

    def baselines_from_ms(self,cfg_par):
        '''
        Reads which baselines were used in the observations
        Stores them sorted by lenght in the array baselines_sort
        Creates the Matrix to analize visibilites on each baseline separately
        '''

        self.logger.info('\t ... Sorting baselines ... \n')


        #sort baselines by length
        baselines = []
        for i in range(0,self.nant-1) :
            for j in range(i+1,self.nant) :
                # Exclude anticorrelations
                if self.aperfi_badant != None:
                    if self.ant_names[i]!=self.ant_names[j] and any(x != self.ant_names[i] for x in self.aperfi_badant) and any(x != self.ant_names[i] for x in self.aperfi_badant):
                        #distances are in units of meters
                        xdiff = (self.ant_pos[i,0]-self.ant_pos[j,0])*(self.ant_pos[i,0]-self.ant_pos[j,0])
                        ydiff = (self.ant_pos[i,1]-self.ant_pos[j,1])*(self.ant_pos[i,1]-self.ant_pos[j,1])
                        zdiff = (self.ant_pos[i,2]-self.ant_pos[j,2])*(self.ant_pos[i,2]-self.ant_pos[j,2])

                        dist = np.sqrt(xdiff+ydiff+zdiff)
                        baselines.append(([self.ant_names[i],self.ant_names[j]],dist))
                else: 
                    if self.ant_names[i]!=self.ant_names[j]:
                        #distances are in units of meters
                        xdiff = (self.ant_pos[i,0]-self.ant_pos[j,0])*(self.ant_pos[i,0]-self.ant_pos[j,0])
                        ydiff = (self.ant_pos[i,1]-self.ant_pos[j,1])*(self.ant_pos[i,1]-self.ant_pos[j,1])
                        zdiff = (self.ant_pos[i,2]-self.ant_pos[j,2])*(self.ant_pos[i,2]-self.ant_pos[j,2])

                        dist = np.sqrt(xdiff+ydiff+zdiff)
                        baselines.append(([self.ant_names[i],self.ant_names[j]],dist))


        self.baselines_sort = sorted(baselines, key=lambda baselines: baselines[1])  

        cfg_par['rfi']['baseline_lenghts'] = np.array(self.baselines_sort)[:,1]
 
        #baseline statistics
        rfiST.baseline_stats(cfg_par)

        # Define matrix of indecese of baselines                                         
        self.blMatrix=np.zeros((self.nant,self.nant),dtype=int)
        for i in range(0,len(self.baselines_sort)) :

            ant1 = self.baselines_sort[i][0][0]
            ant2 = self.baselines_sort[i][0][1]
            self.blMatrix[ant1,ant2] = i
            self.blMatrix[ant2,ant1] = i


        self.logger.info('\t ... Matrix of baselines sorted by length loaded ...\n\n')


    def priors_flag(self,cfg_par):
        '''

        Flags YY,XY,YX polarizations
        Flags autocorrelations
        Flags bad antennas set by aperfi_badant = [ x, y, z ]
        Stores information on baselines sorted by lenght in baselines_sort 
        Stores visibilities in a cube (baselines[sorted by length], frequency, time_scans of each baseline)

        '''

        self.logger.info('\t ... Flagging a-prioris  ...\n')
        pol = cfg_par['rfi']['polarization']

        #if (pol == 'q' or pol == 'Q' or pol == 'i' or pol != 'I') and ( cfg_par['rfi']['RFInder_mode'] == 'use_flags'):
        #   self.datacube = np.zeros([len(self.baselines_sort),2*self.flag.shape[1],2*self.flag.shape[0]/(len(self.baselines_sort))])
        #else:
        self.datacube = np.zeros([len(self.baselines_sort),self.flag.shape[1],self.flag.shape[0]/(len(self.baselines_sort))])

        baseline_counter = np.zeros((self.nant,self.nant),dtype=int)
        #flag unused polarizations
        if (pol == 'xx' or pol == 'XX'):
            self.flag[:,:,1] = True #YY
            if self.flag.shape[2]>2:
                self.flag[:,:,2] = True #XY
                self.flag[:,:,3] = True #YX
        elif (pol == 'yy' or pol == 'YY'):
            self.flag[:,:,0] = True #YY
            self.flag[:,:,2] = True #XY
            self.flag[:,:,3] = True #YX
        elif (pol == 'xy' or pol == 'XY'):
            self.flag[:,:,0] = True #YY
            self.flag[:,:,1] = True #XY
            self.flag[:,:,3] = True #YX      
        elif (pol == 'yx' or pol == 'YX'):
            self.flag[:,:,0] = True #YY
            self.flag[:,:,1] = True #XY
            self.flag[:,:,2] = True #YX  
        elif (pol == 'q' or pol == 'QQ'):
            if self.flag.shape[2]>2:
                self.flag[:,:,2] = True #XY
                self.flag[:,:,3] = True #YX


        #flag autocorrelations and bad antennas
        for i in xrange(0,self.flag.shape[0]):

            if self.aperfi_badant != None:
                if (any(x == self.ant1[i] for x in self.aperfi_badant) or any(x == self.ant2[i] for x in self.aperfi_badant)):
                    self.flag[i,:,0] = True
            if self.ant1[i] == self.ant2[i]:
                self.flag[i,:,0] = True            
            else:
                a1 = self.ant1[i]
                a2 = self.ant2[i]
                indice=self.blMatrix[a1,a2]
                # Count how many visibilities per baseline
                counter=baseline_counter[a1,a2]
                # Put amplitude of visibility
                # In the right place in the new array
                if cfg_par['rfi']['RFInder_mode'] == 'use_flags':
                    if (pol == 'xx' or pol == 'XX'):
                        self.datacube[indice,:,counter]=self.flag[i,:,0]
                    if (pol == 'yy' or pol == 'YY'):
                        self.datacube[indice,:,counter]=self.flag[i,:,1]
                    if (pol == 'xy' or pol == 'XY'):
                        self.datacube[indice,:,counter]=self.flag[i,:,2]
                    if (pol == 'yx' or pol == 'YX'):
                        self.datacube[indice,:,counter]=self.flag[i,:,3]
                    elif (pol == 'q' or pol == 'Q' or pol == 'I' or pol == 'i'):
                        self.datacube[indice,:,counter]=np.multiply(self.flag[i,:,0],self.flag[i,:,1])                 
                elif cfg_par['rfi']['RFInder_mode'] == 'rms_clip':
                    if (pol == 'xx' or pol == 'XX'):
                        self.datacube[indice,:,counter]=np.abs(self.vis[i,:,0])
                    if (pol == 'yy' or pol == 'YY'):
                        self.datacube[indice,:,counter]=np.abs(self.vis[i,:,1])
                    if (pol == 'xy' or pol == 'XY'):
                        self.datacube[indice,:,counter]=np.abs(self.vis[i,:,2])
                    if (pol == 'yx' or pol == 'YX'):
                        self.datacube[indice,:,counter]=np.abs(self.vis[i,:,3])
                    elif (pol == 'q' or pol == 'Q'):
                        self.datacube[indice,:,counter]=np.abs(self.vis[i,:,0])-np.abs(self.vis[i,:,1])/2.
                # Update the number of visibility in that baseline
                baseline_counter[a1,a2]+=1

        #why is this here?
        #self.datacube = np.transpose(self.datacube, (1, 0, 2)) 

 

        self.logger.info('\t ... Prior flagging done ...\n\n')

        return self.datacube

    def find_rfi(self,datas,cfg_par,time_step=-1):
        '''

        For each baseline finds all signal above rms*aperfi_clip
        Creates a cube of visibilities sorted by baseline_lengh , frequency, time
        Stores them sorted by lenght in the array baselines_sort
        Creates the Matrix to analize visibilites on each baseline separately

        '''
        
        self.logger.info('\t ... Searching for RFI ...\n')
       
        self.datacube = datas

        rms = np.zeros([self.datacube.shape[0],self.datacube.shape[1]])
        self.mean_array = np.zeros([self.datacube.shape[0],self.datacube.shape[1]])
        self.flag_lim_array= np.zeros([self.datacube.shape[0]])

        self.aperfi_rfifree_min = float(cfg_par['rfi']['noise_measure_edges'][0])

        if (self.aperfi_rfifree_min < float(cfg_par['rfi']['lowfreq']) or self.aperfi_rfifree_min > float(cfg_par['rfi']['highfreq'])):
            self.aperfi_rfifree_min = float(cfg_par['rfi']['lowfreq'])
            self.logger.warning('\t ### Minimum frequency for RFI free range outside of observed interval ')
            self.logger.warning('\t     Correct noise_measure_edges in rfi of parameter file ###')

        self.aperfi_rfifree_max = float(cfg_par['rfi']['noise_measure_edges'][1])
        if (self.aperfi_rfifree_max < float(cfg_par['rfi']['lowfreq']) or self.aperfi_rfifree_max > float(cfg_par['rfi']['highfreq'])):
            self.aperfi_rfifree_max = float(cfg_par['rfi']['highfreq'])
            self.logger.warning('\t ### Maximum frequency for RFI free range outside of observed interval ')
            self.logger.warning('\t     Correct noise_measure_edges in rfi of parameter file ###')


        self.aperfi_rmsclip = float(cfg_par['rfi']['rms_clip'])


        chan_min = np.argmin(np.abs(self.channelFreqs[0] - self.aperfi_rfifree_min))
        chan_max = np.argmin(np.abs(self.channelFreqs[0] - self.aperfi_rfifree_max))

        time_ax_len = int(self.datacube.shape[2])

        if cfg_par['rfi']['RFInder_mode'] == 'rms_clip': 

            for i in xrange(0,self.datacube.shape[0]):
                
                if self.datacube.shape[2] == 0:
                    rms[i,:] = 100.
                else:
                    tmp_rms = np.nanmedian(self.datacube[i, chan_min:chan_max, 0])
                    med2 = abs(self.datacube[i, chan_min:chan_max, 0] - tmp_rms)
                    madfm = np.ma.median(med2) / 0.6744888
                    flag_lim = self.aperfi_rmsclip*madfm  
                
                    self.flag_lim_array[i] = flag_lim    

                    for j in xrange(0,self.datacube.shape[1]):
                        tmpar = self.datacube[i,j,:]
                        mean  = np.nanmean(tmpar)
                        tmpar = tmpar-mean
                        tmpar = abs(tmpar)
                        self.mean_array[i,j] = mean
                        #change masked values to very high number
                        #inds = np.where(np.isnan(tmpar))
                        tmpar[np.isnan(tmpar)]=np.inf
                        tmpar.sort()
                        index_rms = np.argmin(np.abs(tmpar - flag_lim))
                        tmp_over = len(tmpar[index_rms:-1])+1
                        if tmp_over == 1. :
                            tmp_over = 0.
                        rms[i,j] = 100.*tmp_over/time_ax_len

        elif cfg_par['rfi']['RFInder_mode'] == 'use_flags': 
            for i in xrange(0,self.datacube.shape[0]):
                for j in xrange(0,self.datacube.shape[1]):
                    rms[i,j] = 100.*np.nansum(self.datacube[i,j,:])/time_ax_len

        rfi_files.write_freq_base(cfg_par,rms,time_step)

        self.logger.info('\t ... RFI found ... \n\n')


        return 0

    def write_freq_base(self,cfg_par,rms,time_step=-1) :
        '''
        
        Writes an image.fits of visibilities ordered by frequency, baseline.
        Baselines are ordered by their length.
        
        '''
        self.logger.info('\t ... Wrtiting RFI by frequency and baseline ...')


        #reverse frequencies if going from high-to-low         
        if time_step != -1:
            time_tmp = int(float(cfg_par['rfi']['chunks']['time_step'])*time_step)
            if time_tmp == 0:
                time_name = '00'+str(time_tmp)+'m'
            elif time_tmp <100:
                time_name = '0'+str(time_tmp)+'m'
            else:
                time_name= str(time_tmp)+'m'    
        else:
            time_name = 'full' 
        
        self.rfi_freq_base = cfg_par['general']['rfidir']+'freq_base_'+time_name+'_im.fits'
        #set fits file
        hdu = fits.PrimaryHDU(rms)
        hdulist = fits.HDUList([hdu])
        header = hdulist[0].header
        #write header keywords               
        header['CRPIX1'] = 1
        header['CDELT1'] = np.abs(self.channelWidths[0,0]/1e6)
        header['CRVAL1'] = self.channelFreqs[0,0]/1e6
        header['CTYPE1'] = ('Frequency')
        header['CUNIT1'] = ('MHz')
        header['CRPIX2'] = 1
        header['CRVAL2'] = 1
        header['CDELT2'] = 1
        header['CTYPE2'] =  ('Baseline')
        header['POLAR'] =  ('XX')
        header['BUNIT'] = ('% > '+str(5)+'*rms')
        header['BTYPE'] = ('intensity')
        #write file
        fits.writeto(self.rfi_freq_base,rms,header,overwrite=True)
        hdulist.close()

        self.logger.info('\t ... RFI on fits file ...\n')


        return 0

    def rfi_flag(self,cfg_par):
        '''
        Creates a new MS file where RFI has been flagged in the FLAG column
        '''

        #self.flag_lim_array = np.zeros([self.vis.shape[0]])

        self.logger.info('\t ... Flagging RFI ... \n')


        for i in xrange(0,self.vis.shape[0]):
            
            if any(x == self.ant1[i] for x in self.aperfi_badant) or any(x == self.ant2[i] for x in self.aperfi_badant):
                continue
            else:
                indice=self.blMatrix[self.ant1[i],self.ant2[i]]
                flag_clip = self.flag_lim_array[indice]
            
            for j in xrange(0,self.vis.shape[1]):
                mean = self.mean_array[indice,i]
                if (np.abs(self.vis[i,j,0]) - mean ) > flag_clip:
                
                    self.flag[i,j,0] = True
        

        self.rfifile = cfg_par['general']['rfidir']+'rfi_flagged_vis.MS'

        t=tables.table(self.msfile)
        tout = t.copy(self.rfifile)       # make deep copy
        tout.close()
        t.close()
        tout = tables.table(self.rfifile, readonly=False)
        #coldmi = tout.getdminfo('DATA')     # get dminfo of existing column
        #coldmi["NAME"] = 'CORRECTED_DATA'               # give it a unique name
        #tout.addcols(tables.maketabdesc(tables.makearrcoldesc("CORRECTED_DATA",0., ndim=2)),
        #           coldmi)
        #tout.putcol('CORRECTED_DATA',vis)
        tout.putcol('FLAG',self.flag)
        tout.close()


        self.logger.info('\t ... RFI flagged ... \n\n')


        return 0




import os
from setuptools import setup, find_packages

pkg = 'rfinder'
build_root = os.path.dirname(__file__)

scripts = ["bin/" + i for i in os.listdir("bin")]


def readme():
    """Get readme content for package long description"""
    with open(os.path.join(build_root, 'README.md')) as f:
        return f.read()


def requirements():
    """Get package requirements"""
    with open(os.path.join(build_root, 'requirements.txt')) as f:
        return [pname.strip() for pname in f.readlines()]

__version__ = "1.0.5"

setup(name=pkg,
      version=__version__,
      description="A tool to identify the presence of RFI in an observation and visualize it",
      long_description=readme(),
      long_description_content_type = 'text/markdown',
      author="Filippo Marcello Maccagni",
      author_email="filippo.maccagni@gmail.com",
      packages=find_packages(),
      include_package_data=True,
      package_data={
          'rfinder': ['rfinder_default.yml',
                      'templates/*.html']
      },
      url="https://github.com/Fil8/RFInder",
      license="GNU GPL 3",
      classifiers=["Intended Audience :: Developers",
                   "Programming Language :: Python :: 2",
                   "Topic :: Scientific/Engineering :: Astronomy",
                   "Topic :: Software Development :: Libraries :: Python Modules"],
      platforms=["OS Independent"],
      install_requires=requirements(),
      scripts=scripts)
